# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vim.option import BoolOption
from pyVmomi.vim.option import IntOption
from pyVmomi.vim.option import LongOption

from pyVmomi.vim.vm.device import VirtualDeviceOption

class VirtualVideoCardOption(VirtualDeviceOption):
   videoRamSizeInKB: Optional[LongOption] = None
   numDisplays: Optional[IntOption] = None
   useAutoDetect: Optional[BoolOption] = None
   support3D: Optional[BoolOption] = None
   use3dRendererSupported: Optional[BoolOption] = None
   graphicsMemorySizeInKB: Optional[LongOption] = None
   graphicsMemorySizeSupported: Optional[BoolOption] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vim import Datastore
from pyVmomi.vim import KeyValue

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.cns import Cursor
from pyVmomi.vim.cns import VolumeId

class QueryFilter(DynamicData):
   volumeIds: list[VolumeId] = []
   names: list[str] = []
   containerClusterIds: list[str] = []
   storagePolicyId: Optional[str] = None
   datastores: list[Datastore] = []
   labels: list[KeyValue] = []
   complianceStatus: Optional[str] = None
   datastoreAccessibilityStatus: Optional[str] = None
   cursor: Optional[Cursor] = None
   healthStatus: Optional[str] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import ClassVar

from pyVmomi.VmomiSupport import Enum

from pyVmomi.vmodl import DynamicData

class FeatureVersionInfo(DynamicData):
   class FeatureVersionKey(Enum):
      faultTolerance: ClassVar['FeatureVersionKey'] = 'faultTolerance'

   key: str
   value: str



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from . import host as host
from . import vcenter as vcenter

from .BaseConfigInfo import BaseConfigInfo as BaseConfigInfo
from .CloneSpec import CloneSpec as CloneSpec
from .CreateSpec import CreateSpec as CreateSpec
from .DiskCryptoSpec import DiskCryptoSpec as DiskCryptoSpec
from .DiskInfoFlag import DiskInfoFlag as DiskInfoFlag
from .ID import ID as ID
from .InfrastructureObjectPolicy import InfrastructureObjectPolicy as InfrastructureObjectPolicy
from .InfrastructureObjectPolicySpec import InfrastructureObjectPolicySpec as InfrastructureObjectPolicySpec
from .MigrateSpec import MigrateSpec as MigrateSpec
from .ReconcileResult import ReconcileResult as ReconcileResult
from .ReconcileSpec import ReconcileSpec as ReconcileSpec
from .RelocateSpec import RelocateSpec as RelocateSpec
from .StateInfo import StateInfo as StateInfo
from .TagEntry import TagEntry as TagEntry
from .VClockInfo import VClockInfo as VClockInfo
from .VStorageObject import VStorageObject as VStorageObject
from .VStorageObjectControlFlag import VStorageObjectControlFlag as VStorageObjectControlFlag
from .VStorageObjectManagerBase import VStorageObjectManagerBase as VStorageObjectManagerBase
from .VStorageObjectSnapshot import VStorageObjectSnapshot as VStorageObjectSnapshot
from .VStorageObjectSnapshotDetails import VStorageObjectSnapshotDetails as VStorageObjectSnapshotDetails
from .VStorageObjectSnapshotInfo import VStorageObjectSnapshotInfo as VStorageObjectSnapshotInfo



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vim.vm.customization import GuiRunOnce
from pyVmomi.vim.vm.customization import GuiUnattended
from pyVmomi.vim.vm.customization import Identification
from pyVmomi.vim.vm.customization import IdentitySettings
from pyVmomi.vim.vm.customization import LicenseFilePrintData
from pyVmomi.vim.vm.customization import UserData

class Sysprep(IdentitySettings):
   guiUnattended: GuiUnattended
   userData: UserData
   guiRunOnce: Optional[GuiRunOnce] = None
   identification: Identification
   licenseFilePrintData: Optional[LicenseFilePrintData] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import NoReturn
from typing import Optional

from pyVmomi.VmomiSupport import ManagedObject
from pyVmomi.VmomiSupport import long

from pyVmomi.vim import VirtualMachine

from pyVmomi.vim.vm.guest import GuestAuthentication

class AuthManager(ManagedObject):
   def ValidateCredentials(self, vm: VirtualMachine, auth: GuestAuthentication) -> NoReturn: ...
   def AcquireCredentials(self, vm: VirtualMachine, requestedAuth: GuestAuthentication, sessionID: Optional[long]) -> GuestAuthentication: ...
   def ReleaseCredentials(self, vm: VirtualMachine, auth: GuestAuthentication) -> NoReturn: ...



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.VmomiSupport import long

from pyVmomi.vim import SharesInfo

from pyVmomi.vmodl import DynamicData

class NetworkResourcePool(DynamicData):
   class AllocationInfo(DynamicData):
      limit: Optional[long] = None
      shares: Optional[SharesInfo] = None
      priorityTag: Optional[int] = None

   class ConfigSpec(DynamicData):
      key: str
      configVersion: Optional[str] = None
      allocationInfo: Optional[AllocationInfo] = None
      name: Optional[str] = None
      description: Optional[str] = None

   key: str
   name: Optional[str] = None
   description: Optional[str] = None
   configVersion: str
   allocationInfo: AllocationInfo



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import ClassVar

from pyVmomi.VmomiSupport import Enum

from pyVmomi.vim.fault import VmConfigFault

class VFlashModuleNotSupported(VmConfigFault):
   class Reason(Enum):
      CacheModeNotSupported: ClassVar['Reason'] = 'CacheModeNotSupported'
      CacheConsistencyTypeNotSupported: ClassVar['Reason'] = 'CacheConsistencyTypeNotSupported'
      CacheBlockSizeNotSupported: ClassVar['Reason'] = 'CacheBlockSizeNotSupported'
      CacheReservationNotSupported: ClassVar['Reason'] = 'CacheReservationNotSupported'
      DiskSizeNotSupported: ClassVar['Reason'] = 'DiskSizeNotSupported'

   vmName: str
   moduleName: str
   reason: str
   hostName: str



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import NoReturn
from typing import Optional

from pyVmomi.VmomiSupport import ManagedObject

from pyVmomi.vim import ManagedEntity
from pyVmomi.vim import TaskInfo

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.event import Event
from pyVmomi.vim.event import EventDescription
from pyVmomi.vim.event import EventFilterSpec
from pyVmomi.vim.event import EventHistoryCollector

class EventManager(ManagedObject):
   class EventViewSpec(DynamicData):
      pass

   class ViewByStartId(EventViewSpec):
      startEventId: int
      isForward: bool

   @property
   def description(self) -> EventDescription: ...
   @property
   def latestEvent(self) -> Optional[Event]: ...
   @property
   def maxCollector(self) -> int: ...

   def RetrieveArgumentDescription(self, eventTypeId: str) -> list[EventDescription.EventArgDesc]: ...
   def CreateCollector(self, filter: EventFilterSpec) -> EventHistoryCollector: ...
   def LogUserEvent(self, entity: ManagedEntity, msg: str) -> NoReturn: ...
   def QueryEvent(self, filter: EventFilterSpec, eventViewSpec: Optional[EventViewSpec]) -> list[Event]: ...
   def PostEvent(self, eventToPost: Event, taskInfo: Optional[TaskInfo]) -> NoReturn: ...



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import NoReturn
from typing import Optional

from pyVmomi.VmomiSupport import ManagedObject

from pyVmomi.vim.encryption import CryptoKeyId
from pyVmomi.vim.encryption import CryptoKeyPlain
from pyVmomi.vim.encryption import CryptoKeyResult

class CryptoManager(ManagedObject):
   @property
   def enabled(self) -> bool: ...

   def AddKey(self, key: CryptoKeyPlain) -> NoReturn: ...
   def AddKeys(self, keys: list[CryptoKeyPlain]) -> list[CryptoKeyResult]: ...
   def RemoveKey(self, key: CryptoKeyId, force: bool) -> NoReturn: ...
   def RemoveKeys(self, keys: list[CryptoKeyId], force: bool) -> list[CryptoKeyResult]: ...
   def ListKeys(self, limit: Optional[int]) -> list[CryptoKeyId]: ...



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vim.vm.device import VirtualDevice

class VirtualUSB(VirtualDevice):
   class USBBackingInfo(VirtualDevice.DeviceBackingInfo):
      pass

   class RemoteHostBackingInfo(VirtualDevice.DeviceBackingInfo):
      hostname: str

   class RemoteClientBackingInfo(VirtualDevice.RemoteDeviceBackingInfo):
      hostname: str

   connected: bool
   vendor: Optional[int] = None
   product: Optional[int] = None
   family: list[str] = []
   speed: list[str] = []



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import ClassVar
from typing import NoReturn
from typing import Optional

from pyVmomi.VmomiSupport import Enum

from pyVmomi.vim.profile import ComplianceProfile
from pyVmomi.vim.profile import Profile

class ClusterProfile(Profile):
   class ConfigInfo(Profile.ConfigInfo):
      complyProfile: Optional[ComplianceProfile] = None

   class CreateSpec(Profile.CreateSpec):
      pass

   class ConfigSpec(CreateSpec):
      pass

   class CompleteConfigSpec(ConfigSpec):
      complyProfile: Optional[ComplianceProfile] = None

   class ServiceType(Enum):
      DRS: ClassVar['ServiceType'] = 'DRS'
      HA: ClassVar['ServiceType'] = 'HA'
      DPM: ClassVar['ServiceType'] = 'DPM'
      FT: ClassVar['ServiceType'] = 'FT'

   class ConfigServiceCreateSpec(ConfigSpec):
      serviceType: list[str] = []

   def Update(self, config: ConfigSpec) -> NoReturn: ...



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import NoReturn
from typing import Optional

from pyVmomi.vim import ExtensibleManagedObject

from pyVmomi.vim.host import GraphicsConfig
from pyVmomi.vim.host import GraphicsInfo
from pyVmomi.vim.host import SharedGpuCapabilities

from pyVmomi.vim.vm import VgpuDeviceInfo
from pyVmomi.vim.vm import VgpuProfileInfo

class GraphicsManager(ExtensibleManagedObject):
   @property
   def graphicsInfo(self) -> list[GraphicsInfo]: ...
   @property
   def graphicsConfig(self) -> Optional[GraphicsConfig]: ...
   @property
   def sharedPassthruGpuTypes(self) -> list[str]: ...
   @property
   def sharedGpuCapabilities(self) -> list[SharedGpuCapabilities]: ...

   def RetrieveVgpuDeviceInfo(self) -> list[VgpuDeviceInfo]: ...
   def RetrieveVgpuProfileInfo(self) -> list[VgpuProfileInfo]: ...
   def Refresh(self) -> NoReturn: ...
   def IsSharedGraphicsActive(self) -> bool: ...
   def UpdateGraphicsConfig(self, config: GraphicsConfig) -> NoReturn: ...



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import ClassVar
from typing import Optional

from pyVmomi.VmomiSupport import Enum

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.host import DnsConfig
from pyVmomi.vim.host import IpRouteConfig
from pyVmomi.vim.host import IpRouteTableConfig

class NetStackInstance(DynamicData):
   class SystemStackKey(Enum):
      defaultTcpipStack: ClassVar['SystemStackKey'] = 'defaultTcpipStack'
      vmotion: ClassVar['SystemStackKey'] = 'vmotion'
      vSphereProvisioning: ClassVar['SystemStackKey'] = 'vSphereProvisioning'
      mirror: ClassVar['SystemStackKey'] = 'mirror'
      ops: ClassVar['SystemStackKey'] = 'ops'

   class CongestionControlAlgorithmType(Enum):
      newreno: ClassVar['CongestionControlAlgorithmType'] = 'newreno'
      cubic: ClassVar['CongestionControlAlgorithmType'] = 'cubic'

   key: Optional[str] = None
   name: Optional[str] = None
   dnsConfig: Optional[DnsConfig] = None
   ipRouteConfig: Optional[IpRouteConfig] = None
   requestedMaxNumberOfConnections: Optional[int] = None
   congestionControlAlgorithm: Optional[str] = None
   ipV6Enabled: Optional[bool] = None
   routeTableConfig: Optional[IpRouteTableConfig] = None
   owner: Optional[str] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vim.vm.customization import IdentitySettings
from pyVmomi.vim.vm.customization import NameGenerator

class LinuxPrep(IdentitySettings):
   hostName: NameGenerator
   domain: str
   timeZone: Optional[str] = None
   hwClockUTC: Optional[bool] = None
   scriptText: Optional[str] = None
   compatibleCustomizationMethod: Optional[str] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.cluster import VsanObjectIdentity
from pyVmomi.vim.cluster import VsanObjectSpaceSummary

from pyVmomi.vim.host import VsanObjectOverallHealth

class VsanObjectIdentityAndHealth(DynamicData):
   identities: list[VsanObjectIdentity] = []
   health: Optional[VsanObjectOverallHealth] = None
   spaceSummary: list[VsanObjectSpaceSummary] = []
   rawData: Optional[str] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from datetime import datetime

from typing import Optional

from pyVmomi.vmodl import DynamicData

class ClusterStatus(DynamicData):
   class State(DynamicData):
      class CompletionEstimate(DynamicData):
         completeTime: Optional[datetime] = None
         percentComplete: Optional[int] = None

      state: str
      completion: Optional[CompletionEstimate] = None

   uuid: Optional[str] = None
   nodeUuid: Optional[str] = None
   health: str
   nodeState: State
   memberUuid: list[str] = []



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import NoReturn
from typing import Optional

from pyVmomi.VmomiSupport import ManagedObject

from pyVmomi.vmodl import DynamicData

class LocalAccountManager(ManagedObject):
   class AccountSpecification(DynamicData):
      id: str
      password: Optional[str] = None
      description: Optional[str] = None

   class PosixAccountSpecification(AccountSpecification):
      posixId: Optional[int] = None
      shellAccess: Optional[bool] = None

   def CreateUser(self, user: AccountSpecification) -> NoReturn: ...
   def UpdateUser(self, user: AccountSpecification) -> NoReturn: ...
   def CreateGroup(self, group: AccountSpecification) -> NoReturn: ...
   def RemoveUser(self, userName: str) -> NoReturn: ...
   def RemoveGroup(self, groupName: str) -> NoReturn: ...
   def AssignUserToGroup(self, user: str, group: str) -> NoReturn: ...
   def UnassignUserFromGroup(self, user: str, group: str) -> NoReturn: ...
   def ChangePassword(self, user: str, oldPassword: str, newPassword: str) -> NoReturn: ...



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.host import HostBusAdapter
from pyVmomi.vim.host import MultipathInfo
from pyVmomi.vim.host import NvmeTopology
from pyVmomi.vim.host import PlugStoreTopology
from pyVmomi.vim.host import ScsiLun
from pyVmomi.vim.host import ScsiTopology

class StorageDeviceInfo(DynamicData):
   hostBusAdapter: list[HostBusAdapter] = []
   scsiLun: list[ScsiLun] = []
   scsiTopology: Optional[ScsiTopology] = None
   nvmeTopology: Optional[NvmeTopology] = None
   multipathInfo: Optional[MultipathInfo] = None
   plugStoreTopology: Optional[PlugStoreTopology] = None
   softwareInternetScsiEnabled: bool



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import ClassVar
from typing import Optional

from pyVmomi.VmomiSupport import Enum

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.vsan.host import MembershipInfo

class VsanRuntimeInfo(DynamicData):
   class DiskIssueType(Enum):
      nonExist: ClassVar['DiskIssueType'] = 'nonExist'
      stampMismatch: ClassVar['DiskIssueType'] = 'stampMismatch'
      unknown: ClassVar['DiskIssueType'] = 'unknown'

   class DiskIssue(DynamicData):
      diskId: str
      issue: str

   membershipList: list[MembershipInfo] = []
   diskIssues: list[DiskIssue] = []
   accessGenNo: Optional[int] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import ClassVar
from typing import NoReturn
from typing import Optional

from pyVmomi.VmomiSupport import Enum
from pyVmomi.VmomiSupport import long

from pyVmomi.vim import ExtensibleManagedObject

from pyVmomi.vmodl import DynamicData

class MemoryManagerSystem(ExtensibleManagedObject):
   class ServiceConsoleReservationInfo(DynamicData):
      serviceConsoleReservedCfg: long
      serviceConsoleReserved: long
      unreserved: long

   class VirtualMachineReservationInfo(DynamicData):
      class AllocationPolicy(Enum):
         swapNone: ClassVar['AllocationPolicy'] = 'swapNone'
         swapSome: ClassVar['AllocationPolicy'] = 'swapSome'
         swapMost: ClassVar['AllocationPolicy'] = 'swapMost'

      virtualMachineMin: long
      virtualMachineMax: long
      virtualMachineReserved: long
      allocationPolicy: str

   class VirtualMachineReservationSpec(DynamicData):
      virtualMachineReserved: Optional[long] = None
      allocationPolicy: Optional[str] = None

   @property
   def consoleReservationInfo(self) -> Optional[ServiceConsoleReservationInfo]: ...
   @property
   def virtualMachineReservationInfo(self) -> Optional[VirtualMachineReservationInfo]: ...

   def ReconfigureServiceConsoleReservation(self, cfgBytes: long) -> NoReturn: ...
   def ReconfigureVirtualMachineReservation(self, spec: VirtualMachineReservationSpec) -> NoReturn: ...



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import ClassVar
from typing import Optional

from pyVmomi.VmomiSupport import Enum

from pyVmomi.vim import KeyValue

from pyVmomi.vmodl import DynamicData
from pyVmomi.vmodl import LocalizableMessage

class VsanClusterHealthAction(DynamicData):
   class VsanClusterHealthActionIdEnum(Enum):
      RepairClusterObjectsAction: ClassVar['VsanClusterHealthActionIdEnum'] = 'RepairClusterObjectsAction'
      UploadHclDb: ClassVar['VsanClusterHealthActionIdEnum'] = 'UploadHclDb'
      UpdateHclDbFromInternet: ClassVar['VsanClusterHealthActionIdEnum'] = 'UpdateHclDbFromInternet'
      EnableHealthService: ClassVar['VsanClusterHealthActionIdEnum'] = 'EnableHealthService'
      DiskBalance: ClassVar['VsanClusterHealthActionIdEnum'] = 'DiskBalance'
      StopDiskBalance: ClassVar['VsanClusterHealthActionIdEnum'] = 'StopDiskBalance'
      RemediateDedup: ClassVar['VsanClusterHealthActionIdEnum'] = 'RemediateDedup'
      UpgradeVsanDiskFormat: ClassVar['VsanClusterHealthActionIdEnum'] = 'UpgradeVsanDiskFormat'
      CreateDVS: ClassVar['VsanClusterHealthActionIdEnum'] = 'CreateDVS'
      ConfigureHA: ClassVar['VsanClusterHealthActionIdEnum'] = 'ConfigureHA'
      ConfigureDRS: ClassVar['VsanClusterHealthActionIdEnum'] = 'ConfigureDRS'
      ConfigureVSAN: ClassVar['VsanClusterHealthActionIdEnum'] = 'ConfigureVSAN'
      ClaimVSANDisks: ClassVar['VsanClusterHealthActionIdEnum'] = 'ClaimVSANDisks'
      ClusterUpgrade: ClassVar['VsanClusterHealthActionIdEnum'] = 'ClusterUpgrade'
      CreateVMKnic: ClassVar['VsanClusterHealthActionIdEnum'] = 'CreateVMKnic'
      CreateVMKnicWithVMotion: ClassVar['VsanClusterHealthActionIdEnum'] = 'CreateVMKnicWithVMotion'
      RunBurnInTest: ClassVar['VsanClusterHealthActionIdEnum'] = 'RunBurnInTest'
      EnableIscsiTargetService: ClassVar['VsanClusterHealthActionIdEnum'] = 'EnableIscsiTargetService'
      EnablePerformanceServiceAction: ClassVar['VsanClusterHealthActionIdEnum'] = 'EnablePerformanceServiceAction'
      RemediateClusterConfig: ClassVar['VsanClusterHealthActionIdEnum'] = 'RemediateClusterConfig'
      EnableCeip: ClassVar['VsanClusterHealthActionIdEnum'] = 'EnableCeip'
      LoginVumIsoDepot: ClassVar['VsanClusterHealthActionIdEnum'] = 'LoginVumIsoDepot'
      PurgeInaccessSwapObjs: ClassVar['VsanClusterHealthActionIdEnum'] = 'PurgeInaccessSwapObjs'
      UploadReleaseCatalog: ClassVar['VsanClusterHealthActionIdEnum'] = 'UploadReleaseCatalog'
      ConfigureAutomaticRebalance: ClassVar['VsanClusterHealthActionIdEnum'] = 'ConfigureAutomaticRebalance'
      RemediateFileService: ClassVar['VsanClusterHealthActionIdEnum'] = 'RemediateFileService'
      RelayoutVsanObjects: ClassVar['VsanClusterHealthActionIdEnum'] = 'RelayoutVsanObjects'
      RemediateFileServiceImbalance: ClassVar['VsanClusterHealthActionIdEnum'] = 'RemediateFileServiceImbalance'
      SelectNvme: ClassVar['VsanClusterHealthActionIdEnum'] = 'SelectNvme'
      CreateFileServiceDomain: ClassVar['VsanClusterHealthActionIdEnum'] = 'CreateFileServiceDomain'
      RemediateIscsiLunsRuntimeStatus: ClassVar['VsanClusterHealthActionIdEnum'] = 'RemediateIscsiLunsRuntimeStatus'
      ShallowRekey: ClassVar['VsanClusterHealthActionIdEnum'] = 'ShallowRekey'
      VsanClusterHealthActionIdEnum_Unknown: ClassVar['VsanClusterHealthActionIdEnum'] = 'VsanClusterHealthActionIdEnum_Unknown'

   actionId: str
   actionLabel: LocalizableMessage
   actionDescription: LocalizableMessage
   enabled: bool
   parameters: list[KeyValue] = []



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from .CannotConfigureSolutions import CannotConfigureSolutions as CannotConfigureSolutions
from .CannotUploadDepot import CannotUploadDepot as CannotUploadDepot
from .DepotIssue import DepotIssue as DepotIssue
from .InaccessibleDepot import InaccessibleDepot as InaccessibleDepot
from .InvalidDepot import InvalidDepot as InvalidDepot
from .PMIssue import PMIssue as PMIssue
from .PMUnavailable import PMUnavailable as PMUnavailable



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.VmomiSupport import long

from pyVmomi.vmodl import DynamicData

class VsanNetworkPeerHealthResult(DynamicData):
   peer: Optional[str] = None
   peerHostname: Optional[str] = None
   peerVmknicName: Optional[str] = None
   smallPingTestSuccessPct: Optional[int] = None
   largePingTestSuccessPct: Optional[int] = None
   maxLatencyUs: Optional[long] = None
   onSameIpSubnet: Optional[bool] = None
   sourceVmknicName: Optional[str] = None
   connectivityHealthState: Optional[str] = None
   missingHeartBeatCount: Optional[int] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vmodl import DynamicData

class VsanFileServerHealthSummary(DynamicData):
   domainName: Optional[str] = None
   fileServerIp: Optional[str] = None
   nfsdHealth: Optional[str] = None
   networkHealth: Optional[str] = None
   rootfsHealth: Optional[str] = None
   description: Optional[str] = None
   smbConnections: Optional[int] = None
   smbDaemonHealth: Optional[str] = None
   adTestJoinHealth: Optional[str] = None
   dnsLookupHealth: Optional[str] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import NoReturn
from typing import Optional

from pyVmomi.VmomiSupport import ManagedObject
from pyVmomi.VmomiSupport import long

from pyVmomi.vim import Datastore
from pyVmomi.vim import KeyValue
from pyVmomi.vim import Task
from pyVmomi.vim import VirtualMachine

from pyVmomi.vim.encryption import CryptoSpec

from pyVmomi.vim.vm import ProfileSpec

from pyVmomi.vim.vslm import CloneSpec
from pyVmomi.vim.vslm import CreateSpec
from pyVmomi.vim.vslm import DiskCryptoSpec
from pyVmomi.vim.vslm import ID
from pyVmomi.vim.vslm import InfrastructureObjectPolicy
from pyVmomi.vim.vslm import InfrastructureObjectPolicySpec
from pyVmomi.vim.vslm import RelocateSpec
from pyVmomi.vim.vslm import StateInfo
from pyVmomi.vim.vslm import TagEntry
from pyVmomi.vim.vslm import VStorageObject
from pyVmomi.vim.vslm import VStorageObjectSnapshotDetails
from pyVmomi.vim.vslm import VStorageObjectSnapshotInfo

from pyVmomi.vslm.vso import DatastoreSyncStatus
from pyVmomi.vslm.vso import VStorageObjectAssociations
from pyVmomi.vslm.vso import VStorageObjectQueryResult
from pyVmomi.vslm.vso import VStorageObjectQuerySpec
from pyVmomi.vslm.vso import VStorageObjectResult

class VStorageObjectManager(ManagedObject):
   def CreateDisk(self, spec: CreateSpec) -> Task: ...
   def RegisterDisk(self, path: str, name: Optional[str]) -> VStorageObject: ...
   def ExtendDisk(self, id: ID, newCapacityInMB: long) -> Task: ...
   def InflateDisk(self, id: ID) -> Task: ...
   def RenameVStorageObject(self, id: ID, name: str) -> NoReturn: ...
   def UpdateVStorageObjectPolicy(self, id: ID, profile: list[ProfileSpec]) -> Task: ...
   def UpdateVStorageObjectCrypto(self, id: ID, profile: list[ProfileSpec], disksCrypto: Optional[DiskCryptoSpec]) -> Task: ...
   def UpdateVStorageInfrastructureObjectPolicy(self, spec: InfrastructureObjectPolicySpec) -> Task: ...
   def RetrieveVStorageInfrastructureObjectPolicy(self, datastore: Datastore) -> list[InfrastructureObjectPolicy]: ...
   def DeleteVStorageObject(self, id: ID) -> Task: ...
   def RetrieveVStorageObject(self, id: ID) -> VStorageObject: ...
   def RetrieveVStorageObjectState(self, id: ID) -> StateInfo: ...
   def RetrieveVStorageObjectAssociations(self, ids: list[ID]) -> list[VStorageObjectAssociations]: ...
   def ListVStorageObjectsForSpec(self, query: list[VStorageObjectQuerySpec], maxResult: int) -> Optional[VStorageObjectQueryResult]: ...
   def CloneVStorageObject(self, id: ID, spec: CloneSpec) -> Task: ...
   def RelocateVStorageObject(self, id: ID, spec: RelocateSpec) -> Task: ...
   def SetVStorageObjectControlFlags(self, id: ID, controlFlags: list[str]) -> NoReturn: ...
   def ClearVStorageObjectControlFlags(self, id: ID, controlFlags: list[str]) -> NoReturn: ...
   def AttachTagToVStorageObject(self, id: ID, category: str, tag: str) -> NoReturn: ...
   def DetachTagFromVStorageObject(self, id: ID, category: str, tag: str) -> NoReturn: ...
   def ListVStorageObjectsAttachedToTag(self, category: str, tag: str) -> list[ID]: ...
   def ListTagsAttachedToVStorageObject(self, id: ID) -> list[TagEntry]: ...
   def ReconcileDatastoreInventory(self, datastore: Datastore) -> Task: ...
   def ScheduleReconcileDatastoreInventory(self, datastore: Datastore) -> NoReturn: ...
   def CreateSnapshot(self, id: ID, description: str) -> Task: ...
   def DeleteSnapshot(self, id: ID, snapshotId: ID) -> Task: ...
   def RetrieveSnapshotInfo(self, id: ID) -> VStorageObjectSnapshotInfo: ...
   def CreateDiskFromSnapshot(self, id: ID, snapshotId: ID, name: str, profile: list[ProfileSpec], crypto: Optional[CryptoSpec], path: Optional[str]) -> Task: ...
   def RevertVStorageObject(self, id: ID, snapshotId: ID) -> Task: ...
   def RetrieveSnapshotDetails(self, id: ID, snapshotId: ID) -> VStorageObjectSnapshotDetails: ...
   def QueryChangedDiskAreas(self, id: ID, snapshotId: ID, startOffset: long, changeId: str) -> VirtualMachine.DiskChangeInfo: ...
   def QueryGlobalCatalogSyncStatus(self) -> list[DatastoreSyncStatus]: ...
   def QueryGlobalCatalogSyncStatusForDatastore(self, datastoreURL: str) -> Optional[DatastoreSyncStatus]: ...
   def UpdateVStorageObjectMetadata(self, id: ID, metadata: list[KeyValue], deleteKeys: list[str]) -> Task: ...
   def RetrieveVStorageObjectMetadata(self, id: ID, snapshotId: Optional[ID], prefix: Optional[str]) -> list[KeyValue]: ...
   def RetrieveVStorageObjectMetadataValue(self, id: ID, snapshotId: Optional[ID], key: str) -> str: ...
   def RetrieveVStorageObjects(self, ids: list[ID]) -> list[VStorageObjectResult]: ...
   def AttachDisk(self, id: ID, vm: VirtualMachine, controllerKey: Optional[int], unitNumber: Optional[int]) -> Task: ...



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vim import Task

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.cluster import DrsFaults
from pyVmomi.vim.cluster import Recommendation

class StoragePlacementResult(DynamicData):
   recommendations: list[Recommendation] = []
   drsFault: Optional[DrsFaults] = None
   task: Optional[Task] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.VmomiSupport import ManagedMethod

from pyVmomi.vim.action import Action
from pyVmomi.vim.action import MethodActionArgument

class MethodAction(Action):
   name: ManagedMethod
   argument: list[MethodActionArgument] = []



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.VmomiSupport import ManagedObject
from pyVmomi.VmomiSupport import long

from pyVmomi.vim import HistoricalInterval

from pyVmomi.vmodl import DynamicData

class ResourcePlanningManager(ManagedObject):
   class DatabaseSizeParam(DynamicData):
      inventoryDesc: InventoryDescription
      perfStatsDesc: Optional[PerfStatsDescription] = None

   class InventoryDescription(DynamicData):
      numHosts: int
      numVirtualMachines: int
      numResourcePools: Optional[int] = None
      numClusters: Optional[int] = None
      numCpuDev: Optional[int] = None
      numNetDev: Optional[int] = None
      numDiskDev: Optional[int] = None
      numvCpuDev: Optional[int] = None
      numvNetDev: Optional[int] = None
      numvDiskDev: Optional[int] = None

   class PerfStatsDescription(DynamicData):
      intervals: list[HistoricalInterval] = []

   class DatabaseSizeEstimate(DynamicData):
      size: long

   def EstimateDatabaseSize(self, dbSizeParam: DatabaseSizeParam) -> DatabaseSizeEstimate: ...



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import NoReturn
from typing import Optional

from pyVmomi.VmomiSupport import ManagedObject

from pyVmomi.vim import ManagedEntity
from pyVmomi.vim import Task

from pyVmomi.vim.profile import ComplianceResult
from pyVmomi.vim.profile import ExpressionMetadata
from pyVmomi.vim.profile import Profile

class ComplianceManager(ManagedObject):
   def CheckCompliance(self, profile: list[Profile], entity: list[ManagedEntity]) -> Task: ...
   def QueryComplianceStatus(self, profile: list[Profile], entity: list[ManagedEntity]) -> list[ComplianceResult]: ...
   def ClearComplianceStatus(self, profile: list[Profile], entity: list[ManagedEntity]) -> NoReturn: ...
   def QueryExpressionMetadata(self, expressionName: list[str], profile: Optional[Profile]) -> list[ExpressionMetadata]: ...



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.cluster import VsanPerfEntityMetricCSV

class VsanPerfDiagnosticResult(DynamicData):
   exceptionId: str
   recommendation: Optional[str] = None
   aggregationFunction: Optional[str] = None
   aggregationData: Optional[VsanPerfEntityMetricCSV] = None
   exceptionData: list[VsanPerfEntityMetricCSV] = []



# Copyright (c) 2006-2024 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from pyVmomi.vmodl import RuntimeFault

class SecurityError(RuntimeFault):
   pass



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vim import OpaqueNetwork

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.option import OptionValue

class OpaqueNetworkInfo(DynamicData):
   opaqueNetworkId: str
   opaqueNetworkName: str
   opaqueNetworkType: str
   pnicZone: list[str] = []
   capability: Optional[OpaqueNetwork.Capability] = None
   extraConfig: list[OptionValue] = []



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from pyVmomi.vim import ElementDescription
from pyVmomi.vim import TypeDescription

from pyVmomi.vmodl import DynamicData

class TaskDescription(DynamicData):
   methodInfo: list[ElementDescription] = []
   state: list[ElementDescription] = []
   reason: list[TypeDescription] = []



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.VmomiSupport import ManagedObject

from pyVmomi.vim import ManagedEntity

from pyVmomi.vim.profile import PolicyMetadata
from pyVmomi.vim.profile import Profile

class ProfileManager(ManagedObject):
   @property
   def profile(self) -> list[Profile]: ...

   def CreateProfile(self, createSpec: Profile.CreateSpec) -> Profile: ...
   def QueryPolicyMetadata(self, policyName: list[str], profile: Optional[Profile]) -> list[PolicyMetadata]: ...
   def FindAssociatedProfile(self, entity: ManagedEntity) -> list[Profile]: ...



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vim import ManagedEntity

from pyVmomi.vmodl import DynamicData

class RuleInfo(DynamicData):
   key: Optional[int] = None
   status: Optional[ManagedEntity.Status] = None
   enabled: Optional[bool] = None
   name: Optional[str] = None
   mandatory: Optional[bool] = None
   userCreated: Optional[bool] = None
   inCompliance: Optional[bool] = None
   ruleUuid: Optional[str] = None



#############################################################
# Copyright (c) 2005-2025 Broadcom. All Rights Reserved.
# The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.
#############################################################

# @file connect.py
# @brief Connect to a VMOMI ServiceInstance.
#
# Detailed description (for Doxygen goes here)
"""
Connect to a VMOMI ServiceInstance.

Detailed description (for [e]pydoc goes here).
"""

import re
import ssl

from sys import exc_info
from xml.parsers.expat import ExpatError

from pyVmomi.five import PY3, HTTPConnection, HTTPSConnection
from pyVmomi import SessionOrientedStub, SoapStubAdapter, vim, vmodl
from pyVmomi.SoapAdapter import CONNECTION_POOL_IDLE_TIMEOUT_SEC
from pyVmomi.VmomiSupport import (GetServiceVersions, IsChildVersion, nsMap,
                                  versionIdMap, versionMap)

try:
    from xml.etree.ElementTree import ElementTree
except ImportError:
    from elementtree.ElementTree import ElementTree


TOKEN_TYPE_OAUTH_BEARER = 'oauth-bearer'
TOKEN_TYPE_SAML = 'saml'
TOKEN_TYPE_SSPI = 'sspi'
TOKEN_TYPES = [TOKEN_TYPE_OAUTH_BEARER, TOKEN_TYPE_SAML, TOKEN_TYPE_SSPI]

"""
Global regular expression for parsing host and port connection
See https://www.ietf.org/rfc/rfc3986.txt sec 3.2.2
"""
_rx = re.compile(r"(^\[.+\]|[^:]+)(:\d+)?$")

_si = None
"""
Global (thread-shared) ServiceInstance

@todo: Get rid of me?
"""


def getSslContext(host, sslContext, disableSslCertValidation):
    """
    Connections to 'localhost' do not need SSL verification as a certificate
    will never match. The OS provides security by only allowing root to bind
    to low-numbered ports.
    """
    """
    TODO: the entire 127.0.0.0/8 network is allocated for loopback addresses.
    Therefore, the check should be for a valid ip4 address, beginning with 127.
    """
    if disableSslCertValidation or (not sslContext and host in ['localhost', '127.0.0.1', '::1']):
        sslContext = ssl._create_unverified_context()
    return sslContext


class closing(object):
    """
    Helper class for using closable objects in a 'with' statement,
    similar to the one provided by contextlib.
    """
    def __init__(self, obj):
        self.obj = obj

    def __enter__(self):
        return self.obj

    def __exit__(self, *exc_info):
        self.obj.close()


class VimSessionOrientedStub(SessionOrientedStub):
    """A vim-specific SessionOrientedStub.  See the SessionOrientedStub class
    in pyVmomi/SoapAdapter.py for more information.
    """

    # The set of exceptions that should trigger a relogin by the session stub.
    SESSION_EXCEPTIONS = (vim.fault.NotAuthenticated, )

    @staticmethod
    def makeUserLoginMethod(username, password, locale=None):
        """Return a function that will call the vim.SessionManager.Login() method
        with the given parameters.  The result of this function can be passed as
        the "loginMethod" to a SessionOrientedStub constructor.
        """
        def _doLogin(soapStub):
            si = vim.ServiceInstance("ServiceInstance", soapStub)
            sm = si.content.sessionManager
            if not sm.currentSession:
                si.content.sessionManager.Login(username, password, locale)

        return _doLogin

    @staticmethod
    def makeExtensionLoginMethod(extensionKey):
        """Return a function that will call the vim.SessionManager.Login() method
        with the given parameters.  The result of this function can be passed as
        the "loginMethod" to a SessionOrientedStub constructor.
        """
        def _doLogin(soapStub):
            si = vim.ServiceInstance("ServiceInstance", soapStub)
            sm = si.content.sessionManager
            if not sm.currentSession:
                si.content.sessionManager.LoginExtensionByCertificate(
                    extensionKey)

        return _doLogin

    @staticmethod
    def makeCertHokTokenLoginMethod(stsUrl, stsCert=None, ssl_context=None):
        """Return a function that will call the vim.SessionManager.LoginByToken()
        after obtaining a HoK SAML token from the STS. The result of this function
        can be passed as the "loginMethod" to a SessionOrientedStub constructor.

        @param stsUrl: URL of the SAML Token issuing service. (i.e. SSO server).
        @param stsCert: public key of the STS service.
        @param ssl_context: SSL context
        """
        assert (stsUrl)

        def _doLogin(soapStub):
            from . import sso
            cert = soapStub.certFile
            key = soapStub.certKeyFile
            authenticator = sso.SsoAuthenticator(sts_url=stsUrl,
                                                 sts_cert=stsCert)

            samlAssertion = authenticator.get_hok_saml_assertion(
                cert, key, ssl_context=ssl_context)

            def _requestModifier(request):
                return sso.add_saml_context(request, samlAssertion, key)

            si = vim.ServiceInstance("ServiceInstance", soapStub)
            sm = si.content.sessionManager
            if not sm.currentSession:
                with soapStub.requestModifier(_requestModifier):
                    try:
                        soapStub.samlToken = samlAssertion
                        si.content.sessionManager.LoginByToken()
                    finally:
                        soapStub.samlToken = None

        return _doLogin

    @staticmethod
    def makeCredBearerTokenLoginMethod(username,
                                       password,
                                       stsUrl,
                                       stsCert=None,
                                       ssl_context=None):
        """Return a function that will call the vim.SessionManager.LoginByToken()
        after obtaining a Bearer token from the STS. The result of this function
        can be passed as the "loginMethod" to a SessionOrientedStub constructor.

        @param username: username of the user/service registered with STS.
        @param password: password of the user/service registered with STS.
        @param stsUrl: URL of the SAML Token issueing service. (i.e. SSO server).
        @param stsCert: public key of the STS service.
        @param ssl_context: SSL context
        """
        assert (username)
        assert (password)
        assert (stsUrl)

        def _doLogin(soapStub):
            from . import sso
            cert = soapStub.certFile
            key = soapStub.certKeyFile
            authenticator = sso.SsoAuthenticator(sts_url=stsUrl,
                                                 sts_cert=stsCert)
            samlAssertion = authenticator.get_bearer_saml_assertion(
                username, password, cert, key, ssl_context=ssl_context)
            si = vim.ServiceInstance("ServiceInstance", soapStub)
            sm = si.content.sessionManager
            if not sm.currentSession:
                try:
                    soapStub.samlToken = samlAssertion
                    si.content.sessionManager.LoginByToken()
                finally:
                    soapStub.samlToken = None

        return _doLogin


def Connect(host='localhost',
            port=443,
            user='root',
            pwd='',
            service="hostd",
            adapter="SOAP",
            namespace=None,
            path="/sdk",
            version=None,
            keyFile=None,
            certFile=None,
            httpProxyHost=None,
            httpProxyPort=80,
            thumbprint=None,
            serverPemCert=None,
            sslContext=None,
            httpConnectionTimeout=None,
            connectionPoolTimeout=CONNECTION_POOL_IDLE_TIMEOUT_SEC,
            token=None,
            tokenType=None,
            disableSslCertValidation=False,
            customHeaders=None,
            sessionId=None,
            # Deprecated
            b64token=None,
            # Deprecated
            mechanism=None):
    """
    Connect to the specified server, login and return the service
    instance object.

    Throws any exception back to caller. The service instance object is
    also saved in the library for easy access.

    Clients should modify the service parameter only when connecting to
    a VMOMI server other than hostd/vpxd. For both of the latter, the
    default value is fine.

    @param host: Which host to connect to.
    @type  host: string
    @param port: Port
    @type  port: int
    @param user: User
    @type  user: string
    @param pwd: Password
    @type  pwd: string
    @param service: Service
    @type  service: string
    @param adapter: Adapter
    @type  adapter: string
    @param namespace: Namespace *** Deprecated: Use version instead ***
    @type  namespace: string
    @param path: Path
    @type  path: string
    @param version: Version
    @type  version: string
    @param keyFile: ssl key file path
    @type  keyFile: string
    @param certFile: ssl cert file path
    @type  certFile: string
    @param httpProxyHost The host name of the proxy server.
    @type  httpProxyHost: string
    @param httpProxyPort The proxy server port.
    @type  httpProxyPort: int
    @param thumbprint: **** Deprecated. Use serverPemCert instead.
                        If both fields are set, thumbprint should match
                        serverPemCert.
                        The SHA1/SHA256/SHA512 thumbprint of the server's
                        SSL certificate.
                        Some use a thumbprint of the form xx:xx:xx..:xx.
                        We ignore the ":" characters.
    @type  thumbprint: string
    @param serverPemCert: PEM-encoded SSL certificate of the
                            host to which we are connecting.
    @type  serverPemCert: string
    @param sslContext: SSL Context describing the various SSL options. It is only
                        supported in Python 2.7.9 or higher.
    @type  sslContext: SSL.Context
    @param httpConnectionTimeout: Timeout in secs for http requests.
    @type  httpConnectionTimeout: int
    @param connectionPoolTimeout: Timeout in secs for idle connections to close, specify
                                    negative numbers for never closing the connections
    @type  connectionPoolTimeout: int
    @type  token: string
    @param token: Authentication and Authorization token to use for the connection.
                    The presence of this token overrides the user and pwd parameters.
    @type  tokenType: string
    @param tokenType: Select which type of Authentication and Authorization token to use.
    @type  disableSslCertValidation: bool
    @param disableSslCertValidation: Creates an unverified SSL context when True.
    @type  customHeaders: dict
    @param customHeaders: Dictionary with custom HTTP headers.
    @type  sessionId: string
    @param sessionId: Allows usage of an existing session.
                        If supplied Login will not be attempted.
    @param b64token: base64 encoded token
           *** Deprecated: Use token instead ***
    @type  b64token: string
    @param mechanism: authentication mechanism: userpass or sspi
           *** Deprecated: Use tokenType instead ***
    @type  mechanism: string
    """
    host, port = parse_hostport(host, port)
    sslContext = getSslContext(host, sslContext, disableSslCertValidation)

    if namespace:
        assert (version is None)
        version = versionMap[namespace]
    elif not version:
        version = "vim.version.version9"

    if b64token or mechanism:
        raise Exception(
            "'b64token' and 'mechanism' are no longer supported. "
            "The default connection type uses credentials. "
            "If you want to authenticate with a token,"
            "set 'token' and 'tokenType'.")

    si, stub = __Login(host,
                       port,
                       user,
                       pwd,
                       service,
                       adapter,
                       version,
                       path,
                       keyFile,
                       certFile,
                       httpProxyHost,
                       httpProxyPort,
                       thumbprint,
                       serverPemCert,
                       sslContext,
                       httpConnectionTimeout,
                       connectionPoolTimeout,
                       token=token,
                       tokenType=tokenType,
                       customHeaders=customHeaders,
                       sessionId=sessionId)
    SetSi(si)

    return si


def Disconnect(si=None):
    """
    Logout and disconnect the service instance
    @param si: The service instance (returned from Connect)
               Defaults to the saved service instance
    """
    if not si:
        si = GetSi()

    if not si:
        return

    try:
        content = si.RetrieveContent()
        content.sessionManager.Logout()
    except Exception as e:
        pass

    si._stub.DropConnections()

    if si == GetSi():
        SetSi(None)


# Method that gets a local ticket for the specified user
def GetLocalTicket(si, user):
    try:
        sessionManager = si.content.sessionManager
    except Exception as e:
        if type(e).__name__ == 'ExpatError':
            msg = 'Malformed response while querying for local ticket: "%s"' % e
            raise vim.fault.HostConnectFault(msg=msg)
        else:
            msg = 'Failed to query for local ticket: "%s"' % e
            raise vim.fault.HostConnectFault(msg=msg)
    localTicket = sessionManager.AcquireLocalTicket(userName=user)
    with open(localTicket.passwordFilePath) as f:
        content = f.read()
    return localTicket.userName, content


# Private method that performs the actual Connect and returns a
# connected service instance object.


def __Login(host,
            port,
            user,
            pwd,
            # TODO Remove service
            service,
            adapter,
            version,
            path,
            keyFile,
            certFile,
            httpProxyHost,
            httpProxyPort,
            thumbprint,
            serverPemCert,
            sslContext,
            httpConnectionTimeout,
            connectionPoolTimeout,
            token,
            tokenType,
            customHeaders,
            sessionId):
    """
    Private method that performs the actual Connect and returns a
    connected service instance object.

    @param host: Which host to connect to.
    @type  host: string
    @param port: Port
    @type  port: int
    @param user: User
    @type  user: string
    @param pwd: Password
    @type  pwd: string
    @param service: Service
    @type  service: string
    @param adapter: Adapter
    @type  adapter: string
    @param version: Version
    @type  version: string
    @param path: Path
    @type  path: string
    @param keyFile: ssl key file path
    @type  keyFile: string
    @param certFile: ssl cert file path
    @type  certFile: string
    @param httpProxyHost The host name of the proxy server.
    @type  httpProxyHost: string
    @param httpProxyPort The proxy server port.
    @type  httpProxyPort: int
    @param thumbprint: **** Deprecated. Use serverPemCert instead.
                        If both fields are set, thumbprint should match
                        serverPemCert.
                        The SHA1/SHA256/SHA512 thumbprint of the server's
                        SSL certificate.
                        Some use a thumbprint of the form xx:xx:xx..:xx.
                        We ignore the ":" characters.
    @type  thumbprint: string
    @param serverPemCert: PEM-encoded SSL certificate of the
                            host to which we are connecting.
    @type  serverPemCert: string
    @param sslContext: SSL Context describing the various SSL options. It is only
                        supported in Python 2.7.9 or higher.
    @type  sslContext: SSL.Context
    @param httpConnectionTimeout: Timeout in secs for http requests.
    @type  httpConnectionTimeout: int
    @param connectionPoolTimeout: Timeout in secs for idle connections to close, specify
                                    negative numbers for never closing the connections
    @type  connectionPoolTimeout: int
    @type  token: string
    @param token: Authentication and Authorization token to use for the connection.
                    The presence of this token overrides the user and pwd parameters.
    @type  tokenType: string
    @param tokenType: Select which type of Authentication and Authorization token to use.
    @type  customHeaders: dict
    @param customHeaders: Dictionary with custom HTTP headers.
    @type  sessionId: string
    @param sessionId: Allows usage of an existing session.
                        If supplied Login will not be attempted.
    """

    # XXX remove the adapter and service arguments once dependent code is fixed
    if adapter != "SOAP":
        raise ValueError(adapter)

    # If a token is provided
    # add OAuth token in HTTP authorization header or
    # add SAML token to SOAP's message wsse header
    samlToken = None
    if token:
        if tokenType == TOKEN_TYPE_OAUTH_BEARER:
            if customHeaders is None:
                customHeaders = {}
            customHeaders.update({"Authorization": "Bearer {}".format(token)})
        elif tokenType == TOKEN_TYPE_SAML:
            samlToken = token

    # Create the SOAP stub adapter
    stub = SoapStubAdapter(
        host,
        port,
        version=version,
        path=path,
        certKeyFile=keyFile,
        certFile=certFile,
        httpProxyHost=httpProxyHost,
        httpProxyPort=httpProxyPort,
        thumbprint=thumbprint,
        serverPemCert=serverPemCert,
        sslContext=sslContext,
        httpConnectionTimeout=httpConnectionTimeout,
        connectionPoolTimeout=connectionPoolTimeout,
        samlToken=samlToken,
        customHeaders=customHeaders,
        sessionId=sessionId)

    # Get Service instance
    si = vim.ServiceInstance("ServiceInstance", stub)
    content = None
    try:
        content = si.RetrieveContent()
    except vmodl.MethodFault:
        raise
    except Exception as e:
        # NOTE (hartsock): preserve the traceback for diagnostics
        # pulling and preserving the traceback makes diagnosing connection
        # failures easier since the fault will also include where inside the
        # library the fault occurred. Without the traceback we have no idea
        # why the connection failed beyond the message string.
        fault = vim.fault.HostConnectFault(msg=str(e))
        if PY3:
            _, _, tb = exc_info()
            if tb:
                raise fault.with_traceback(tb)
        raise fault

    if sessionId:
        return si, stub

    # Get a ticket if we're connecting to localhost and password is not specified
    if host == 'localhost' and not pwd and not token:
        try:
            (user, pwd) = GetLocalTicket(si, user)
        except:
            pass  # This is not supported against vCenter, and connecting
            # with an empty password is fine in debug builds

    # Login
    if not token:
        content.sessionManager.Login(user, pwd, None)
    else:
        if tokenType in [TOKEN_TYPE_OAUTH_BEARER, TOKEN_TYPE_SAML]:
            content.sessionManager.LoginByToken()
        elif tokenType == TOKEN_TYPE_SSPI:
            content.sessionManager.LoginBySSPI(token)
        else:
            raise Exception("'{0}' token type is not supported. "
                            "Supported types are: {1}".format(tokenType, TOKEN_TYPES))
    return si, stub


# Get the saved service instance.


def GetSi():
    """ Get the saved service instance. """
    return _si


# Set the saved service instance.


def SetSi(si):
    """ Set the saved service instance. """

    global _si
    _si = si


# Get the global saved stub


def GetStub():
    """ Get the global saved stub. """
    si = GetSi()
    if si:
        return si._GetStub()
    return None


# RAII-style class for managing connections


class Connection(object):
    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        self.si = None

    def __enter__(self):
        self.si = Connect(*self.args, **self.kwargs)
        return self.si

    def __exit__(self, *exc_info):
        if self.si:
            Disconnect(self.si)
            self.si = None


class SmartConnection(object):
    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        self.si = None

    def __enter__(self):
        self.si = SmartConnect(*self.args, **self.kwargs)
        return self.si

    def __exit__(self, *exc_info):
        if self.si:
            Disconnect(self.si)
            self.si = None


def __GetElementTree(protocol, server, port, path, sslContext,
                     httpProxyHost, httpProxyPort,
                     customHeaders):
    """
    Private method that returns ElementTree for a remote XML document.

    @param protocol: What protocol to use for the connection (e.g. https or http).
    @type  protocol: string
    @param server: Which server to connect to.
    @type  server: string
    @param port: Port
    @type  port: int
    @param path: Path
    @type  path: string
    @param sslContext: SSL Context describing the various SSL options. It is only
                        supported in Python 2.7.9 or higher.
    @type  sslContext: SSL.Context
    @param httpProxyHost The host name of the proxy server.
    @type  httpProxyHost: string
    @param httpProxyPort The proxy server port.
    @type  httpProxyPort: int
    @type  customHeaders: dict
    @param customHeaders: Dictionary with custom HTTP headers.
    """
    tree = ElementTree()
    headers = customHeaders if customHeaders else {}

    if httpProxyHost:
        kwargs = {"context": sslContext} if sslContext else {}
        conn = HTTPSConnection(httpProxyHost, port=httpProxyPort, **kwargs)
        conn.set_tunnel(server, port, headers)
    elif protocol == "https":
        kwargs = {"context": sslContext} if sslContext else {}
        conn = HTTPSConnection(server, port=port, **kwargs)
    elif protocol == "http":
        conn = HTTPConnection(server, port=port)
    else:
        raise Exception("Protocol " + protocol + " not supported.")
    conn.request(method="GET", url=path, headers=headers)
    try:
        response = conn.getresponse()
        if response.status == 200:
            try:
                tree.parse(response)
                return tree
            except ExpatError:
                pass
        return None
    finally:
        conn.close()


# Private method that returns an ElementTree describing the API versions
# supported by the specified server.  The result will be vimServiceVersions.xml
# if it exists, otherwise None.


def __GetServiceVersionDescription(protocol, server, port, path, sslContext,
                                   httpProxyHost, httpProxyPort,
                                   customHeaders):
    """
    Private method that returns an ElementTree describing the API versions
    supported by the specified server.  The result will be vimServiceVersions.xml
    if it exists, otherwise vimService.wsdl if it exists, otherwise None.

    @param protocol: What protocol to use for the connection (e.g. https or http).
    @type  protocol: string
    @param server: Which server to connect to.
    @type  server: string
    @param port: Port
    @type  port: int
    @param path: Path
    @type  path: string
    @param sslContext: SSL Context describing the various SSL options. It is only
                        supported in Python 2.7.9 or higher.
    @type  sslContext: SSL.Context
    @param httpProxyHost The host name of the proxy server.
    @type  httpProxyHost: string
    @param httpProxyPort The proxy server port.
    @type  httpProxyPort: int
    @type  customHeaders: dict
    @param customHeaders: Dictionary with custom HTTP headers.
    """

    return __GetElementTree(protocol, server, port,
                            path + "/vimServiceVersions.xml", sslContext,
                            httpProxyHost, httpProxyPort, customHeaders)


# Private method that returns true if the service version description document
#  indicates that the desired version is supported


def __VersionIsSupported(desiredVersion, serviceVersionDescription):
    """
    Private method that returns true if the service version description document
    indicates that the desired version is supported

    @param desiredVersion: The version we want to see if the server supports
                            (eg. vim.version.version9.
    @type  desiredVersion: string
    @param serviceVersionDescription: An ElementTree for vimServiceVersions.xml
                                        or vimService.wsdl.
    @type  serviceVersionDescription: ElementTree
    """

    root = serviceVersionDescription.getroot()
    if root.tag == 'namespaces':
        # serviceVersionDescription appears to be a vimServiceVersions.xml document
        if root.get('version') != '1.0':
            raise RuntimeError('vimServiceVersions.xml has version {0},'
                               ' which is not understood'
                               .format(root.get('version')))
        desiredVersionId = versionIdMap[desiredVersion]
        supportedVersion = None
        for namespace in root.findall('namespace'):
            versionId = namespace.findtext('version')
            if versionId == desiredVersionId:
                return True
            else:
                for versionId in namespace.findall('priorVersions/version'):
                    if versionId.text == desiredVersionId:
                        return True
    return False


# Private method that returns the most preferred API version supported by the
# specified server,


def __FindSupportedVersion(protocol, server, port, path, preferredApiVersions,
                           sslContext, httpProxyHost, httpProxyPort,
                           customHeaders):
    """
    Private method that returns the most preferred API version supported by the
    specified server,

    @param protocol: What protocol to use for the connection (e.g. https or http).
    @type  protocol: string
    @param server: Which server to connect to.
    @type  server: string
    @param port: Port
    @type  port: int
    @param path: Path
    @type  path: string
    @param preferredApiVersions: Acceptable API version(s) (e.g. vim.version.version9)
                                    If a list of versions is specified the versions should
                                    be ordered from most to least preferred.
    @type  preferredApiVersions: string or string list
    @param sslContext: SSL Context describing the various SSL options. It is only
                        supported in Python 2.7.9 or higher.
    @type  sslContext: SSL.Context
    @param httpProxyHost The host name of the proxy server.
    @type  httpProxyHost: string
    @param httpProxyPort The proxy server port.
    @type  httpProxyPort: int
    @type  customHeaders: dict
    @param customHeaders: Dictionary with custom HTTP headers.
    """

    serviceVersionDescription = __GetServiceVersionDescription(
        protocol, server, port, path, sslContext, httpProxyHost,
        httpProxyPort, customHeaders)
    if serviceVersionDescription is None:
        return None

    if not isinstance(preferredApiVersions, list):
        preferredApiVersions = [preferredApiVersions]

    for desiredVersion in preferredApiVersions:
        if __VersionIsSupported(desiredVersion, serviceVersionDescription):
            return desiredVersion
    return None


def SmartStubAdapter(host='localhost',
                     port=443,
                     path='/sdk',
                     url=None,
                     sock=None,
                     poolSize=5,
                     certFile=None,
                     certKeyFile=None,
                     httpProxyHost=None,
                     httpProxyPort=80,
                     sslProxyPath=None,
                     thumbprint=None,
                     serverPemCert=None,
                     cacertsFile=None,
                     preferredApiVersions=None,
                     acceptCompressedResponses=True,
                     samlToken=None,
                     sslContext=None,
                     httpConnectionTimeout=None,
                     connectionPoolTimeout=CONNECTION_POOL_IDLE_TIMEOUT_SEC,
                     disableSslCertValidation=False,
                     customHeaders=None,
                     sessionId=None):
    """
    Determine the most preferred API version supported by the specified server,
    then create a soap stub adapter using that version

    The parameters are the same as for pyVmomi.SoapStubAdapter except for
    version which is renamed to prefferedApiVersions

    @param preferredApiVersions: Acceptable API version(s) (e.g. vim.version.version9)
                                    If a list of versions is specified the versions should
                                    be ordered from most to least preferred.  If None is
                                    specified, the list of versions support by pyVmomi will
                                    be used.
    @type  preferredApiVersions: string or string list
    @type  disableSslCertValidation: bool
    @param disableSslCertValidation: Creates an unverified SSL context when True.
    @type  customHeaders: dict
    @param customHeaders: Dictionary with custom HTTP headers.
    @type  sessionId: string
    @param sessionId: Allows usage of an existing session.
                        If supplied Login will not be attempted.
    """
    if preferredApiVersions is None:
        preferredApiVersions = GetServiceVersions('vim25')

    host, port = parse_hostport(host, port)
    sslContext = getSslContext(host, sslContext, disableSslCertValidation)

    supportedVersion = __FindSupportedVersion('https' if port > 0 else 'http',
                                              host, port, path,
                                              preferredApiVersions, sslContext,
                                              httpProxyHost, httpProxyPort,
                                              customHeaders)
    if supportedVersion is None:
        raise Exception("{0}:{1} is down or is not a VIM server"
                        .format(host, port))

    return SoapStubAdapter(host=host,
                           port=port,
                           path=path,
                           url=url,
                           sock=sock,
                           poolSize=poolSize,
                           certFile=certFile,
                           certKeyFile=certKeyFile,
                           httpProxyHost=httpProxyHost,
                           httpProxyPort=httpProxyPort,
                           sslProxyPath=sslProxyPath,
                           thumbprint=thumbprint,
                           serverPemCert=serverPemCert,
                           cacertsFile=cacertsFile,
                           version=supportedVersion,
                           acceptCompressedResponses=acceptCompressedResponses,
                           samlToken=samlToken,
                           sslContext=sslContext,
                           httpConnectionTimeout=httpConnectionTimeout,
                           connectionPoolTimeout=connectionPoolTimeout,
                           customHeaders=customHeaders,
                           sessionId=sessionId)


def SmartConnect(protocol='https',
                 host='localhost',
                 port=443,
                 user='root',
                 pwd='',
                 service="hostd",
                 path="/sdk",
                 preferredApiVersions=None,
                 keyFile=None,
                 certFile=None,
                 httpProxyHost=None,
                 httpProxyPort=80,
                 thumbprint=None,
                 serverPemCert=None,
                 sslContext=None,
                 httpConnectionTimeout=None,
                 connectionPoolTimeout=CONNECTION_POOL_IDLE_TIMEOUT_SEC,
                 token=None,
                 tokenType=None,
                 disableSslCertValidation=False,
                 customHeaders=None,
                 sessionId=None,
                 # Deprecated
                 b64token=None,
                 # Deprecated
                 mechanism=None):
    """
    Determine the most preferred API version supported by the specified server,
    then connect to the specified server using that API version, login and return
    the service instance object.

    Throws any exception back to caller. The service instance object is
    also saved in the library for easy access.

    Clients should modify the service parameter only when connecting to
    a VMOMI server other than hostd/vpxd. For both of the latter, the
    default value is fine.

    @param protocol: What protocol to use for the connection (e.g. https or http).
    @type  protocol: string
    @param host: Which host to connect to.
    @type  host: string
    @param port: Port
    @type  port: int
    @param user: User
    @type  user: string
    @param pwd: Password
    @type  pwd: string
    @param service: Service
    @type  service: string
    @param path: Path
    @type  path: string
    @param preferredApiVersions: Acceptable API version(s) (e.g. vim.version.version9)
                                If a list of versions is specified the versions should
                                be ordered from most to least preferred.  If None is
                                specified, the list of versions support by pyVmomi will
                                be used.
    @type  preferredApiVersions: string or string list
    @param keyFile: ssl key file path
    @type  keyFile: string
    @param certFile: ssl cert file path
    @type  certFile: string
    @param httpProxyHost The host name of the proxy server.
    @type  httpProxyHost: string
    @param httpProxyPort The proxy server port.
    @type  httpProxyPort: int
    @param thumbprint: **** Deprecated. Use serverPemCert instead.
                        If both fields are set, thumbprint should match
                        serverPemCert.
                        The SHA1/SHA256/SHA512 thumbprint of the server's
                        SSL certificate.
                        Some use a thumbprint of the form xx:xx:xx..:xx.
                        We ignore the ":" characters.
    @type  thumbprint: string
    @param serverPemCert: PEM-encoded SSL certificate of the
                            host to which we are connecting.
    @type  serverPemCert: string
    @param sslContext: SSL Context describing the various SSL options. It is only
                        supported in Python 2.7.9 or higher.
    @type  sslContext: SSL.Context
    @param httpConnectionTimeout: Timeout in secs for http requests.
    @type  httpConnectionTimeout: int
    @param connectionPoolTimeout: Timeout in secs for idle connections to close, specify
                                    negative numbers for never closing the connections
    @type  connectionPoolTimeout: int
    @type  token: string
    @param token: Authentication and Authorization token to use for the connection.
                    The presence of this token overrides the user and pwd parameters.
    @type disableSslCertValidation: bool
    @param disableSslCertValidation: Creates an unverified SSL context when True.
    @type  customHeaders: dict
    @param customHeaders: Dictionary with custom HTTP headers.
    @type  sessionId: string
    @param sessionId: Allows usage of an existing session.
                        If supplied Login will not be attempted.
    @param b64token: base64 encoded token
           *** Deprecated: Use token instead ***
    @type  b64token: string
    @param mechanism: authentication mechanism: userpass or sspi
           *** Deprecated: Use tokenType instead ***
    @type  mechanism: string
    """

    if preferredApiVersions is None:
        preferredApiVersions = GetServiceVersions('vim25')

    host, port = parse_hostport(host, port)
    sslContext = getSslContext(host, sslContext, disableSslCertValidation)

    supportedVersion = __FindSupportedVersion(protocol, host, port, path,
                                              preferredApiVersions, sslContext,
                                              httpProxyHost, httpProxyPort,
                                              customHeaders)
    if supportedVersion is None:
        raise Exception("{0}:{1} is down or is not a VIM server"
                        .format(host, port))

    portNumber = protocol == "http" and -int(port) or int(port)

    return Connect(host=host,
                   port=portNumber,
                   user=user,
                   pwd=pwd,
                   service=service,
                   adapter='SOAP',
                   version=supportedVersion,
                   path=path,
                   keyFile=keyFile,
                   certFile=certFile,
                   httpProxyHost=httpProxyHost,
                   httpProxyPort=httpProxyPort,
                   thumbprint=thumbprint,
                   serverPemCert=serverPemCert,
                   sslContext=sslContext,
                   httpConnectionTimeout=httpConnectionTimeout,
                   connectionPoolTimeout=connectionPoolTimeout,
                   token=token,
                   tokenType=tokenType,
                   disableSslCertValidation=disableSslCertValidation,
                   customHeaders=customHeaders,
                   sessionId=sessionId,
                   b64token=b64token,
                   mechanism=mechanism)


def IsManagedHost():
    """
    Check whether the host is managed by vCenter
    """
    try:
        SmartConnect()
        return False
    except Exception as e:
        # connect to local server will be refused when host managed by vCenter
        return True


def parse_hostport(host, port):
    try:
        info = re.match(_rx, host)
        if info is not None:
            host = info.group(1)
            if host[0] == '[':
                host = info.group(1)[1:-1]
            if info.group(2) is not None:
                port = int(info.group(2)[1:])
    except ValueError as ve:
        pass

    return host, port



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.VmomiSupport import long

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.encryption import KmipServerSpec

class EncryptionInfo(DynamicData):
   enabled: Optional[bool] = None
   kekId: Optional[str] = None
   hostKeyId: Optional[str] = None
   kmipServers: list[KmipServerSpec] = []
   kmsServerCerts: list[str] = []
   clientKey: Optional[str] = None
   clientCert: Optional[str] = None
   dekGenerationId: Optional[long] = None
   changing: Optional[bool] = None
   eraseDisksBeforeUse: Optional[bool] = None
   wrappedDek: Optional[str] = None
   dekId: Optional[str] = None
   oldWrappedDek: Optional[str] = None
   oldDekId: Optional[str] = None
   kekVerifier: Optional[str] = None
   dekVerifier: Optional[str] = None
   oldDekVerifier: Optional[str] = None
   iv: Optional[str] = None
   syncing: Optional[bool] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from .ArrayUpdateSpec import ArrayUpdateSpec as ArrayUpdateSpec
from .BoolOption import BoolOption as BoolOption
from .ChoiceOption import ChoiceOption as ChoiceOption
from .FloatOption import FloatOption as FloatOption
from .IntOption import IntOption as IntOption
from .LongOption import LongOption as LongOption
from .OptionDef import OptionDef as OptionDef
from .OptionManager import OptionManager as OptionManager
from .OptionType import OptionType as OptionType
from .OptionValue import OptionValue as OptionValue
from .StringOption import StringOption as StringOption



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vim import Network

from pyVmomi.vmodl import DynamicData

class IpPool(DynamicData):
   class IpPoolConfigInfo(DynamicData):
      subnetAddress: Optional[str] = None
      netmask: Optional[str] = None
      gateway: Optional[str] = None
      range: Optional[str] = None
      dns: list[str] = []
      dhcpServerAvailable: Optional[bool] = None
      ipPoolEnabled: Optional[bool] = None

   class Association(DynamicData):
      network: Optional[Network] = None
      networkName: str

   id: Optional[int] = None
   name: Optional[str] = None
   ipv4Config: Optional[IpPoolConfigInfo] = None
   ipv6Config: Optional[IpPoolConfigInfo] = None
   dnsDomain: Optional[str] = None
   dnsSearchPath: Optional[str] = None
   hostPrefix: Optional[str] = None
   httpProxy: Optional[str] = None
   networkAssociation: list[Association] = []
   availableIpv4Addresses: Optional[int] = None
   availableIpv6Addresses: Optional[int] = None
   allocatedIpv4Addresses: Optional[int] = None
   allocatedIpv6Addresses: Optional[int] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import ClassVar
from typing import Optional

from pyVmomi.VmomiSupport import Enum

from pyVmomi.pbm import ExtendedElementDescription

from pyVmomi.vmodl import DynamicData

class LineOfServiceInfo(DynamicData):
   class LineOfServiceEnum(Enum):
      INSPECTION: ClassVar['LineOfServiceEnum'] = 'INSPECTION'
      COMPRESSION: ClassVar['LineOfServiceEnum'] = 'COMPRESSION'
      ENCRYPTION: ClassVar['LineOfServiceEnum'] = 'ENCRYPTION'
      REPLICATION: ClassVar['LineOfServiceEnum'] = 'REPLICATION'
      CACHING: ClassVar['LineOfServiceEnum'] = 'CACHING'
      PERSISTENCE: ClassVar['LineOfServiceEnum'] = 'PERSISTENCE'
      DATA_PROVIDER: ClassVar['LineOfServiceEnum'] = 'DATA_PROVIDER'
      DATASTORE_IO_CONTROL: ClassVar['LineOfServiceEnum'] = 'DATASTORE_IO_CONTROL'
      DATA_PROTECTION: ClassVar['LineOfServiceEnum'] = 'DATA_PROTECTION'
      STRETCHED_CLUSTER: ClassVar['LineOfServiceEnum'] = 'STRETCHED_CLUSTER'

   lineOfService: str
   name: ExtendedElementDescription
   description: Optional[ExtendedElementDescription] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import ClassVar
from typing import Optional

from pyVmomi.VmomiSupport import Enum

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.host import NvmeTransportParameters

class NvmeDiscoveryLog(DynamicData):
   class SubsystemType(Enum):
      discovery: ClassVar['SubsystemType'] = 'discovery'
      nvm: ClassVar['SubsystemType'] = 'nvm'

   class TransportRequirements(Enum):
      secureChannelRequired: ClassVar['TransportRequirements'] = 'secureChannelRequired'
      secureChannelNotRequired: ClassVar['TransportRequirements'] = 'secureChannelNotRequired'
      requirementsNotSpecified: ClassVar['TransportRequirements'] = 'requirementsNotSpecified'

   class Entry(DynamicData):
      subnqn: str
      subsystemType: str
      subsystemPortId: int
      controllerId: int
      adminQueueMaxSize: int
      transportParameters: NvmeTransportParameters
      transportRequirements: str
      connected: bool

   entry: list[Entry] = []
   complete: bool



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vmodl import DynamicData

class ActiveDirectorySpec(DynamicData):
   class Specification(DynamicData):
      domainName: Optional[str] = None
      userName: Optional[str] = None
      password: Optional[str] = None
      camServer: Optional[str] = None
      thumbprint: Optional[str] = None
      certificate: Optional[str] = None
      smartCardAuthenticationEnabled: Optional[bool] = None
      smartCardTrustAnchors: list[str] = []

   changeOperation: str
   spec: Optional[Specification] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.VmomiSupport import ManagedObject
from pyVmomi.VmomiSupport import long

from pyVmomi.vim import Datastore
from pyVmomi.vim import Task

from pyVmomi.vmodl import DynamicData

class CacheConfigurationManager(ManagedObject):
   class CacheConfigurationSpec(DynamicData):
      datastore: Datastore
      swapSize: long

   class CacheConfigurationInfo(DynamicData):
      key: Datastore
      swapSize: long

   @property
   def cacheConfigurationInfo(self) -> list[CacheConfigurationInfo]: ...

   def ConfigureCache(self, spec: CacheConfigurationSpec) -> Task: ...



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from .DeviceId import DeviceId as DeviceId
from .FailoverParam import FailoverParam as FailoverParam
from .FailoverSuccessResult import FailoverSuccessResult as FailoverSuccessResult
from .FaultDomainInfo import FaultDomainInfo as FaultDomainInfo
from .GroupErrorResult import GroupErrorResult as GroupErrorResult
from .GroupInfo import GroupInfo as GroupInfo
from .GroupOperationResult import GroupOperationResult as GroupOperationResult
from .PointInTimeReplicaId import PointInTimeReplicaId as PointInTimeReplicaId
from .PromoteParam import PromoteParam as PromoteParam
from .QueryPointInTimeReplicaParam import QueryPointInTimeReplicaParam as QueryPointInTimeReplicaParam
from .QueryPointInTimeReplicaSuccessResult import QueryPointInTimeReplicaSuccessResult as QueryPointInTimeReplicaSuccessResult
from .QueryPointInTimeReplicaSummaryResult import QueryPointInTimeReplicaSummaryResult as QueryPointInTimeReplicaSummaryResult
from .QueryReplicationGroupSuccessResult import QueryReplicationGroupSuccessResult as QueryReplicationGroupSuccessResult
from .QueryReplicationPeerResult import QueryReplicationPeerResult as QueryReplicationPeerResult
from .RecoveredTargetGroupMemberInfo import RecoveredTargetGroupMemberInfo as RecoveredTargetGroupMemberInfo
from .ReplicaId import ReplicaId as ReplicaId
from .ReplicationState import ReplicationState as ReplicationState
from .ReverseReplicationSuccessResult import ReverseReplicationSuccessResult as ReverseReplicationSuccessResult
from .SourceGroupInfo import SourceGroupInfo as SourceGroupInfo
from .SourceGroupMemberInfo import SourceGroupMemberInfo as SourceGroupMemberInfo
from .SyncReplicationGroupSuccessResult import SyncReplicationGroupSuccessResult as SyncReplicationGroupSuccessResult
from .TargetGroupInfo import TargetGroupInfo as TargetGroupInfo
from .TargetGroupMemberInfo import TargetGroupMemberInfo as TargetGroupMemberInfo
from .TestFailoverParam import TestFailoverParam as TestFailoverParam
from .VVolId import VVolId as VVolId
from .VirtualDiskId import VirtualDiskId as VirtualDiskId
from .VirtualDiskKey import VirtualDiskKey as VirtualDiskKey
from .VirtualDiskMoId import VirtualDiskMoId as VirtualDiskMoId
from .VirtualMachineFilePath import VirtualMachineFilePath as VirtualMachineFilePath
from .VirtualMachineId import VirtualMachineId as VirtualMachineId
from .VirtualMachineMoId import VirtualMachineMoId as VirtualMachineMoId
from .VirtualMachineUUID import VirtualMachineUUID as VirtualMachineUUID



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vmodl import DynamicData
from pyVmomi.vmodl import KeyAnyValue

from pyVmomi.vim.vsan import IOLatencyMetrics

class IOLatency(DynamicData):
   latencyType: str
   sourceEntityUuid: str
   destEntityUuid: str
   readLatencyStats: IOLatencyMetrics
   writeLatencyStats: IOLatencyMetrics
   detailedInfo: list[KeyAnyValue] = []



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import ClassVar
from typing import Optional

from pyVmomi.VmomiSupport import Enum

from pyVmomi.vmodl import DynamicData

class MountInfo(DynamicData):
   class AccessMode(Enum):
      readWrite: ClassVar['AccessMode'] = 'readWrite'
      readOnly: ClassVar['AccessMode'] = 'readOnly'

   class InaccessibleReason(Enum):
      AllPathsDown_Start: ClassVar['InaccessibleReason'] = 'AllPathsDown_Start'
      AllPathsDown_Timeout: ClassVar['InaccessibleReason'] = 'AllPathsDown_Timeout'
      PermanentDeviceLoss: ClassVar['InaccessibleReason'] = 'PermanentDeviceLoss'

   class MountFailedReason(Enum):
      CONNECT_FAILURE: ClassVar['MountFailedReason'] = 'CONNECT_FAILURE'
      MOUNT_NOT_SUPPORTED: ClassVar['MountFailedReason'] = 'MOUNT_NOT_SUPPORTED'
      NFS_NOT_SUPPORTED: ClassVar['MountFailedReason'] = 'NFS_NOT_SUPPORTED'
      MOUNT_DENIED: ClassVar['MountFailedReason'] = 'MOUNT_DENIED'
      MOUNT_NOT_DIR: ClassVar['MountFailedReason'] = 'MOUNT_NOT_DIR'
      VOLUME_LIMIT_EXCEEDED: ClassVar['MountFailedReason'] = 'VOLUME_LIMIT_EXCEEDED'
      CONN_LIMIT_EXCEEDED: ClassVar['MountFailedReason'] = 'CONN_LIMIT_EXCEEDED'
      MOUNT_EXISTS: ClassVar['MountFailedReason'] = 'MOUNT_EXISTS'
      OTHERS: ClassVar['MountFailedReason'] = 'OTHERS'

   path: Optional[str] = None
   accessMode: str
   mounted: Optional[bool] = None
   accessible: Optional[bool] = None
   inaccessibleReason: Optional[str] = None
   vmknicName: Optional[str] = None
   vmknicActive: Optional[bool] = None
   mountFailedReason: Optional[str] = None
   numTcpConnections: Optional[int] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import NoReturn
from typing import Optional

from pyVmomi.vim import ExtensibleManagedObject
from pyVmomi.vim import Task

from pyVmomi.vmodl import DynamicData
from pyVmomi.vmodl import MethodFault

from pyVmomi.vim.host import DiskPartitionInfo
from pyVmomi.vim.host import FcoeConfig
from pyVmomi.vim.host import FileSystemVolumeInfo
from pyVmomi.vim.host import HbaCreateSpec
from pyVmomi.vim.host import InternetScsiHba
from pyVmomi.vim.host import MultipathInfo
from pyVmomi.vim.host import MultipathStateInfo
from pyVmomi.vim.host import NasVolume
from pyVmomi.vim.host import NvmeConnectSpec
from pyVmomi.vim.host import NvmeDisconnectSpec
from pyVmomi.vim.host import NvmeDiscoverSpec
from pyVmomi.vim.host import NvmeDiscoveryLog
from pyVmomi.vim.host import PathSelectionPolicyOption
from pyVmomi.vim.host import ScsiDisk
from pyVmomi.vim.host import StorageArrayTypePolicyOption
from pyVmomi.vim.host import StorageDeviceInfo
from pyVmomi.vim.host import UnresolvedVmfsResolutionResult
from pyVmomi.vim.host import UnresolvedVmfsResolutionSpec
from pyVmomi.vim.host import UnresolvedVmfsVolume
from pyVmomi.vim.host import VffsVolume
from pyVmomi.vim.host import VmfsVolume

class StorageSystem(ExtensibleManagedObject):
   class VmfsVolumeResult(DynamicData):
      key: str
      fault: Optional[MethodFault] = None

   class ScsiLunResult(DynamicData):
      key: str
      fault: Optional[MethodFault] = None

   class DiskLocatorLedResult(DynamicData):
      key: str
      fault: MethodFault

   @property
   def storageDeviceInfo(self) -> Optional[StorageDeviceInfo]: ...
   @property
   def fileSystemVolumeInfo(self) -> FileSystemVolumeInfo: ...
   @property
   def systemFile(self) -> list[str]: ...
   @property
   def multipathStateInfo(self) -> Optional[MultipathStateInfo]: ...

   def RetrieveDiskPartitionInfo(self, devicePath: list[str]) -> list[DiskPartitionInfo]: ...
   def ComputeDiskPartitionInfo(self, devicePath: str, layout: DiskPartitionInfo.Layout, partitionFormat: Optional[str]) -> DiskPartitionInfo: ...
   def ComputeDiskPartitionInfoForResize(self, partition: ScsiDisk.Partition, blockRange: DiskPartitionInfo.BlockRange, partitionFormat: Optional[str]) -> DiskPartitionInfo: ...
   def UpdateDiskPartitions(self, devicePath: str, spec: DiskPartitionInfo.Specification) -> NoReturn: ...
   def FormatVmfs(self, createSpec: VmfsVolume.Specification) -> VmfsVolume: ...
   def MountVmfsVolume(self, vmfsUuid: str) -> NoReturn: ...
   def UnmountVmfsVolume(self, vmfsUuid: str) -> NoReturn: ...
   def UnmountVmfsVolumeEx(self, vmfsUuid: list[str]) -> Task: ...
   def MountVmfsVolumeEx(self, vmfsUuid: list[str]) -> Task: ...
   def UnmapVmfsVolumeEx(self, vmfsUuid: list[str]) -> Task: ...
   def DeleteVmfsVolumeState(self, vmfsUuid: str) -> NoReturn: ...
   def RescanVmfs(self) -> NoReturn: ...
   def AttachVmfsExtent(self, vmfsPath: str, extent: ScsiDisk.Partition) -> NoReturn: ...
   def ExpandVmfsExtent(self, vmfsPath: str, extent: ScsiDisk.Partition) -> NoReturn: ...
   def UpgradeVmfs(self, vmfsPath: str) -> NoReturn: ...
   def UpgradeVmLayout(self) -> NoReturn: ...
   def QueryUnresolvedVmfsVolume(self) -> list[UnresolvedVmfsVolume]: ...
   def ResolveMultipleUnresolvedVmfsVolumes(self, resolutionSpec: list[UnresolvedVmfsResolutionSpec]) -> list[UnresolvedVmfsResolutionResult]: ...
   def ResolveMultipleUnresolvedVmfsVolumesEx(self, resolutionSpec: list[UnresolvedVmfsResolutionSpec]) -> Task: ...
   def UnmountForceMountedVmfsVolume(self, vmfsUuid: str) -> NoReturn: ...
   def RescanHba(self, hbaDevice: str) -> NoReturn: ...
   def RescanAllHba(self) -> NoReturn: ...
   def UpdateSoftwareInternetScsiEnabled(self, enabled: bool) -> NoReturn: ...
   def UpdateInternetScsiDiscoveryProperties(self, iScsiHbaDevice: str, discoveryProperties: InternetScsiHba.DiscoveryProperties) -> NoReturn: ...
   def UpdateInternetScsiAuthenticationProperties(self, iScsiHbaDevice: str, authenticationProperties: InternetScsiHba.AuthenticationProperties, targetSet: Optional[InternetScsiHba.TargetSet]) -> NoReturn: ...
   def UpdateInternetScsiDigestProperties(self, iScsiHbaDevice: str, targetSet: Optional[InternetScsiHba.TargetSet], digestProperties: InternetScsiHba.DigestProperties) -> NoReturn: ...
   def UpdateInternetScsiAdvancedOptions(self, iScsiHbaDevice: str, targetSet: Optional[InternetScsiHba.TargetSet], options: list[InternetScsiHba.ParamValue]) -> NoReturn: ...
   def UpdateInternetScsiIPProperties(self, iScsiHbaDevice: str, ipProperties: InternetScsiHba.IPProperties) -> NoReturn: ...
   def UpdateInternetScsiName(self, iScsiHbaDevice: str, iScsiName: str) -> NoReturn: ...
   def UpdateInternetScsiAlias(self, iScsiHbaDevice: str, iScsiAlias: str) -> NoReturn: ...
   def AddInternetScsiSendTargets(self, iScsiHbaDevice: str, targets: list[InternetScsiHba.SendTarget]) -> NoReturn: ...
   def RemoveInternetScsiSendTargets(self, iScsiHbaDevice: str, targets: list[InternetScsiHba.SendTarget], force: Optional[bool]) -> NoReturn: ...
   def AddInternetScsiStaticTargets(self, iScsiHbaDevice: str, targets: list[InternetScsiHba.StaticTarget]) -> NoReturn: ...
   def RemoveInternetScsiStaticTargets(self, iScsiHbaDevice: str, targets: list[InternetScsiHba.StaticTarget]) -> NoReturn: ...
   def EnableMultipathPath(self, pathName: str) -> NoReturn: ...
   def DisableMultipathPath(self, pathName: str) -> NoReturn: ...
   def SetMultipathLunPolicy(self, lunId: str, policy: MultipathInfo.LogicalUnitPolicy) -> NoReturn: ...
   def UpdateHppMultipathLunPolicy(self, lunId: str, policy: MultipathInfo.HppLogicalUnitPolicy) -> NoReturn: ...
   def QueryPathSelectionPolicyOptions(self) -> list[PathSelectionPolicyOption]: ...
   def QueryStorageArrayTypePolicyOptions(self) -> list[StorageArrayTypePolicyOption]: ...
   def UpdateScsiLunDisplayName(self, lunUuid: str, displayName: str) -> NoReturn: ...
   def DetachScsiLun(self, lunUuid: str) -> NoReturn: ...
   def DetachScsiLunEx(self, lunUuid: list[str]) -> Task: ...
   def DeleteScsiLunState(self, lunCanonicalName: str) -> NoReturn: ...
   def AttachScsiLun(self, lunUuid: str) -> NoReturn: ...
   def AttachScsiLunEx(self, lunUuid: list[str]) -> Task: ...
   def Refresh(self) -> NoReturn: ...
   def DiscoverFcoeHbas(self, fcoeSpec: FcoeConfig.FcoeSpecification) -> NoReturn: ...
   def MarkForRemoval(self, hbaName: str, remove: bool) -> NoReturn: ...
   def FormatVffs(self, createSpec: VffsVolume.Specification) -> VffsVolume: ...
   def ExtendVffs(self, vffsPath: str, devicePath: str, spec: Optional[DiskPartitionInfo.Specification]) -> NoReturn: ...
   def DestroyVffs(self, vffsPath: str) -> NoReturn: ...
   def MountVffsVolume(self, vffsUuid: str) -> NoReturn: ...
   def UnmountVffsVolume(self, vffsUuid: str) -> NoReturn: ...
   def DeleteVffsVolumeState(self, vffsUuid: str) -> NoReturn: ...
   def RescanVffs(self) -> NoReturn: ...
   def QueryAvailableSsds(self, vffsPath: Optional[str]) -> list[ScsiDisk]: ...
   def SetNFSUser(self, user: str, password: str) -> NoReturn: ...
   def ChangeNFSUserPassword(self, password: str) -> NoReturn: ...
   def QueryNFSUser(self) -> Optional[NasVolume.UserInfo]: ...
   def ClearNFSUser(self) -> NoReturn: ...
   def TurnDiskLocatorLedOn(self, scsiDiskUuids: list[str]) -> Task: ...
   def TurnDiskLocatorLedOff(self, scsiDiskUuids: list[str]) -> Task: ...
   def MarkAsSsd(self, scsiDiskUuid: str) -> Task: ...
   def MarkAsNonSsd(self, scsiDiskUuid: str) -> Task: ...
   def MarkAsLocal(self, scsiDiskUuid: str) -> Task: ...
   def MarkAsNonLocal(self, scsiDiskUuid: str) -> Task: ...
   def UpdateVmfsUnmapPriority(self, vmfsUuid: str, unmapPriority: str) -> NoReturn: ...
   def UpdateVmfsUnmapBandwidth(self, vmfsUuid: str, unmapBandwidthSpec: VmfsVolume.UnmapBandwidthSpec) -> NoReturn: ...
   def QueryVmfsConfigOption(self) -> list[VmfsVolume.ConfigOption]: ...
   def MarkPerenniallyReserved(self, lunUuid: str, state: bool) -> NoReturn: ...
   def MarkPerenniallyReservedEx(self, lunUuid: list[str], state: bool) -> Task: ...
   def CreateNvmeOverRdmaAdapter(self, rdmaDeviceName: str) -> NoReturn: ...
   def RemoveNvmeOverRdmaAdapter(self, hbaDeviceName: str) -> NoReturn: ...
   def CreateSoftwareAdapter(self, spec: HbaCreateSpec) -> NoReturn: ...
   def RemoveSoftwareAdapter(self, hbaDeviceName: str) -> NoReturn: ...
   def DiscoverNvmeControllers(self, discoverSpec: NvmeDiscoverSpec) -> NvmeDiscoveryLog: ...
   def ConnectNvmeController(self, connectSpec: NvmeConnectSpec) -> NoReturn: ...
   def DisconnectNvmeController(self, disconnectSpec: NvmeDisconnectSpec) -> NoReturn: ...
   def ConnectNvmeControllerEx(self, connectSpec: list[NvmeConnectSpec]) -> Task: ...
   def DisconnectNvmeControllerEx(self, disconnectSpec: list[NvmeDisconnectSpec]) -> Task: ...



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import ClassVar
from typing import Optional

from pyVmomi.VmomiSupport import Enum

from pyVmomi.vmodl import DynamicData

class VirtualPMem(DynamicData):
   class SnapshotMode(Enum):
      independent_persistent: ClassVar['SnapshotMode'] = 'independent_persistent'
      independent_eraseonrevert: ClassVar['SnapshotMode'] = 'independent_eraseonrevert'

   snapshotMode: Optional[str] = None



# -*- coding: utf-8 -*-

# VMware vSphere Python SDK tests
#
# Copyright (c) 2008-2024 Broadcom. All Rights Reserved.
# The term "Broadcom" refers to Broadcom Inc. and/or its subsidiaries.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import tests

from pyVim import connect
from pyVmomi import vim
from pyVmomi.VmomiJSONEncoder import VmomiJSONEncoder, templateOf

import atexit
import inspect
import json
import os


class JSONTests(tests.VCRTestBase):
    @property
    def datacenter(self):
        return getattr(vim, 'Datacenter')('datacenter-3', self.si._stub)
        
    @property
    def datastore(self):
        return getattr(vim, 'Datastore')('datastore-17', self.si._stub)
        
    @property
    def host(self):
        return getattr(vim, 'HostSystem')('host-14', self.si._stub)

    @property
    def network(self):
        return getattr(vim, 'Network')('network-13', self.si._stub)
        
    @property
    def si(self):
        if not hasattr(self, '_si'):
            self._si = connect.SmartConnect(
                host='vcenter', user='my_user', pwd='my_pass', disableSslCertValidation=True)
            atexit.register(connect.Disconnect, self._si)
        return self._si

    @property
    def vm(self):
        return getattr(vim, 'VirtualMachine')('vm-19', self.si._stub)

    @property
    def vm2(self):
        return getattr(vim, 'VirtualMachine')('vm-20', self.si._stub)

    def expect(self, data):
        """
        Handle results expectation.  If the expect data file does not
        exist we write one out.  Otherwise we read and compare.  This
        means it is the responsibility of the individual creating the
        .expect files to ensure they are correct.
        """
        testname = inspect.stack()[1][3]
        expectfile = 'tests/files/{0}.expect'.format(testname)
        if os.path.exists(expectfile):
            with open(expectfile, 'r') as file:
                expectdata = file.read()
            self.assertEqual(data, expectdata)
        else:
            with open(expectfile, 'w') as file:
                file.write(data)

    # NOTE: sort_keys is needed for expect comparison to work;
    #                 not required for consumption

    # Explodes the VM
    # By definition if the input is a ManagedObject then it explodes.
    @tests.VCRTestBase.my_vcr.use_cassette(
        'test_json_vm_explode_default.yaml',
        cassette_library_dir=tests.fixtures_path, record_mode='once', decode_compressed_response=True)
    def test_json_vm_explode_default(self):
        raw = json.dumps(self.vm, cls=VmomiJSONEncoder, sort_keys=True)
        self.expect(raw)
        # Basic sanity check
        data = json.loads(raw)
        self.assertEqual(data['_vimid'], 'vm-19')
        self.assertEqual(data['_vimref'], 'vim.VirtualMachine:vm-19')
        self.assertEqual(data['_vimtype'], 'vim.VirtualMachine')
        self.assertEqual(data['overallStatus'], 'green')
        self.assertEqual(len(data['network']), 0)
        self.assertEqual(len(data['capability']['dynamicProperty']), 0)
        self.assertIsNone(data['capability']['dynamicType'])

    # Explodes the VM disabling the dynamic field stripping
    # By definition if the input is a ManagedObject then it explodes.
    @tests.VCRTestBase.my_vcr.use_cassette(
        'test_json_vm_explode_strip_dynamic.yaml',
        cassette_library_dir=tests.fixtures_path, record_mode='once', decode_compressed_response=True)
    def test_json_vm_explode_strip_dynamic(self):
        raw = json.dumps(self.vm, cls=VmomiJSONEncoder, sort_keys=True,
                         strip_dynamic=True)
        self.expect(raw)
        # Basic sanity check
        data = json.loads(raw)
        self.assertEqual(data['_vimid'], 'vm-19')
        self.assertEqual(data['_vimref'], 'vim.VirtualMachine:vm-19')
        self.assertEqual(data['_vimtype'], 'vim.VirtualMachine')
        self.assertEqual(data['overallStatus'], 'green')
        self.assertEqual(len(data['network']), 0)
        self.assertTrue('dynamicProperty' not in data['capability'])
        self.assertTrue('dynamicType' not in data['capability'])

    # Explodes the VM and the VM's networks
    # Here self.vm is redundant (see above) but not harmful.
    @tests.VCRTestBase.my_vcr.use_cassette(
        'test_json_vm_explode_objs.yaml',
        cassette_library_dir=tests.fixtures_path, record_mode='once', decode_compressed_response=True)
    def test_json_vm_explode_objs_match(self):
        to_explode = [self.vm]
        for item in self.vm.network:
            to_explode.append(item)
        self.expect(json.dumps(self.vm, cls=VmomiJSONEncoder, sort_keys=True,
                               explode=to_explode))

    # Explodes by type: all VirtualMachine and all of its snapshots
    @tests.VCRTestBase.my_vcr.use_cassette(
        'test_json_vm_explode_type.yaml',
        cassette_library_dir=tests.fixtures_path, record_mode='once', decode_compressed_response=True)
    def test_json_vm_explode_type_match(self):
        self.expect(json.dumps([self.vm, self.vm2], cls=VmomiJSONEncoder,
                               sort_keys=True,
                               explode=[templateOf('VirtualMachine'),
                                        templateOf('VirtualMachineSnapshot')]))

    # Test Datacenter
    @tests.VCRTestBase.my_vcr.use_cassette(
        'test_json_datacenter_explode.yaml',
        cassette_library_dir=tests.fixtures_path, record_mode='once', decode_compressed_response=True)
    def test_json_datacenter_explode(self):
          self.expect(json.dumps(self.datacenter, cls=VmomiJSONEncoder,
                                 sort_keys=True))

    # Test Datastore
    @tests.VCRTestBase.my_vcr.use_cassette(
        'test_json_datastore_explode.yaml',
        cassette_library_dir=tests.fixtures_path, record_mode='once', decode_compressed_response=True)
    def test_json_datastore_explode(self):
          self.expect(json.dumps(self.datastore, cls=VmomiJSONEncoder,
                                 sort_keys=True))

    # Test HostSystem
    @tests.VCRTestBase.my_vcr.use_cassette(
        'test_json_host_explode.yaml',
        cassette_library_dir=tests.fixtures_path, record_mode='once', decode_compressed_response=True)
    def test_json_host_explode(self):
          self.expect(json.dumps(self.host, cls=VmomiJSONEncoder,
                                 sort_keys=True))

    # Test Network
    @tests.VCRTestBase.my_vcr.use_cassette(
        'test_json_network_explode.yaml',
        cassette_library_dir=tests.fixtures_path, record_mode='once', decode_compressed_response=True)
    def test_json_network_explode(self):
          self.expect(json.dumps(self.network, cls=VmomiJSONEncoder,
                                 sort_keys=True))



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.vsan import VsanHclDeviceConstraint
from pyVmomi.vim.vsan import VsanHclMinFwConstraint

class VsanHclDiskConstraint(DynamicData):
   productId: str
   vendor: str
   constraints: list[VsanHclMinFwConstraint] = []
   pcieConstraints: list[VsanHclDeviceConstraint] = []
   partNumber: Optional[str] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.VmomiSupport import ManagedObject

from pyVmomi.vim import ComputeResource
from pyVmomi.vim import Task

from pyVmomi.vim.host import MaintenanceSpec
from pyVmomi.vim.host import VsanHclFirmwareUpdateSpec

from pyVmomi.vim.vsan import VsanVibInstallPreflightStatus
from pyVmomi.vim.vsan import VsanVibScanResult
from pyVmomi.vim.vsan import VsanVibSpec

class VsanUpdateManager(ManagedObject):
   def VsanVibScan(self, cluster: Optional[ComputeResource], vibSpecs: list[VsanVibSpec]) -> list[VsanVibScanResult]: ...
   def VsanVibInstall(self, cluster: Optional[ComputeResource], vibSpecs: list[VsanVibSpec], scanResults: list[VsanVibScanResult], firmwareSpecs: list[VsanHclFirmwareUpdateSpec], maintenanceSpec: Optional[MaintenanceSpec], rolling: Optional[bool], noSigCheck: Optional[bool]) -> Task: ...
   def VsanVibInstallPreflightCheck(self, cluster: Optional[ComputeResource]) -> VsanVibInstallPreflightStatus: ...



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import ClassVar
from typing import Optional

from pyVmomi.VmomiSupport import Enum

from pyVmomi.vim.event import Event

from pyVmomi.vim.fault import VimFault

class DasConfigFault(VimFault):
   class DasConfigFaultReason(Enum):
      HostNetworkMisconfiguration: ClassVar['DasConfigFaultReason'] = 'HostNetworkMisconfiguration'
      HostMisconfiguration: ClassVar['DasConfigFaultReason'] = 'HostMisconfiguration'
      InsufficientPrivileges: ClassVar['DasConfigFaultReason'] = 'InsufficientPrivileges'
      NoPrimaryAgentAvailable: ClassVar['DasConfigFaultReason'] = 'NoPrimaryAgentAvailable'
      Other: ClassVar['DasConfigFaultReason'] = 'Other'
      NoDatastoresConfigured: ClassVar['DasConfigFaultReason'] = 'NoDatastoresConfigured'
      CreateConfigVvolFailed: ClassVar['DasConfigFaultReason'] = 'CreateConfigVvolFailed'
      VSanNotSupportedOnHost: ClassVar['DasConfigFaultReason'] = 'VSanNotSupportedOnHost'
      DasNetworkMisconfiguration: ClassVar['DasConfigFaultReason'] = 'DasNetworkMisconfiguration'
      SetDesiredImageSpecFailed: ClassVar['DasConfigFaultReason'] = 'SetDesiredImageSpecFailed'
      ApplyHAVibsOnClusterFailed: ClassVar['DasConfigFaultReason'] = 'ApplyHAVibsOnClusterFailed'

   reason: Optional[str] = None
   output: Optional[str] = None
   event: list[Event] = []



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.dvs import VmwareDistributedVirtualSwitch

from pyVmomi.vim.vsan import VsanVdsPgMigrationHostInfo

class VsanVdsPgMigrationSpec(DynamicData):
   vssPgName: str
   dvPgName: str
   vdsPgSetting: VmwareDistributedVirtualSwitch.VmwarePortConfigPolicy
   vdsPgType: str
   hosts: list[VsanVdsPgMigrationHostInfo] = []
   collisionRename: bool



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vim import HostSystem
from pyVmomi.vim import ManagedEntity

from pyVmomi.vmodl import DynamicData

class PortCriteria(DynamicData):
   connected: Optional[bool] = None
   active: Optional[bool] = None
   uplinkPort: Optional[bool] = None
   nsxPort: Optional[bool] = None
   scope: Optional[ManagedEntity] = None
   portgroupKey: list[str] = []
   inside: Optional[bool] = None
   portKey: list[str] = []
   host: list[HostSystem] = []



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vmodl import DynamicData

class VimVasaProvider(DynamicData):
   class StatePerArray(DynamicData):
      priority: int
      arrayId: str
      active: bool

   class VirtualHostConfig(DynamicData):
      vhostName: Optional[str] = None
      serviceHost: str
      servicePort: Optional[int] = None

   uid: Optional[str] = None
   url: str
   name: Optional[str] = None
   selfSignedCertificate: Optional[str] = None
   vhostConfig: Optional[VirtualHostConfig] = None
   versionId: Optional[int] = None



# Copyright (c) 2005-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

version_info = (
    9,
    0,
    0,
    0,
)

version_info_str = ".".join(str(v) for v in version_info)

from . import _init_utils  # noqa: E402
from . import VmomiSupport  # noqa: E402
from ._typeinfos import load_typeinfos  # noqa: E402
load_typeinfos()

# All data object types and fault types have DynamicData as an ancestor
# As well load it proactively.
# Note: This should be done before importing SoapAdapter as it uses
# some fault types
VmomiSupport.GetVmodlType("vmodl.DynamicData")

from .SoapAdapter import (  # noqa: E402,F401
    SessionOrientedStub, SoapStubAdapter, StubAdapterBase)

# Deprecated
# ThumbprintMismatchException was originally part of SoapAdapter and
# not in a separate Security module. The insertion into SoapAdapter
# and the top-level pyvmomi module was for backwards compatibility and
# is deprecated since pyVmomi 8.0.3
class ThumbprintMismatchExceptionIsNoMore:
    def __init__(self, *args, **kwargs):
        raise Exception("pyVmomi.ThumbprintMismatchException and"
                        " pyVmomi.SoapAdapter.ThumbprintMismatchException"
                        " are deprecated."
                        " Use pyVmomi.Security.ThumbprintMismatchException"
                        " instead.")

ThumbprintMismatchException = ThumbprintMismatchExceptionIsNoMore
setattr(SoapAdapter, 'ThumbprintMismatchException',
        ThumbprintMismatchExceptionIsNoMore)

types = VmomiSupport.types

# This will allow files to use Create** functions
# directly from pyVmomi
CreateEnumType = VmomiSupport.CreateEnumType
CreateDataType = VmomiSupport.CreateDataType
CreateManagedType = VmomiSupport.CreateManagedType

# For all the top level names, creating a LazyModule object
# in the global namespace of pyVmomi. Files can just import the
# top level namespace and we will figure out what to load and when
# Examples:
# ALLOWED: from pyVmomi import vim
# NOT ALLOWED: from pyVmomi import vim.host
_globals = globals()
for name in VmomiSupport._topLevelNames:
    # TODO: remove when support for upper-case module names is dropped
    upperCaseName = VmomiSupport.Capitalize(name)
    obj = VmomiSupport.LazyModule(name)
    _globals[name] = obj
    if VmomiSupport._allowCapitalizedNames:
        _globals[upperCaseName] = obj
    if not hasattr(VmomiSupport.types, name):
        setattr(VmomiSupport.types, name, obj)
        if VmomiSupport._allowCapitalizedNames:
            setattr(VmomiSupport.types, upperCaseName, obj)
del _globals



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from datetime import datetime

from typing import NoReturn
from typing import Optional

from pyVmomi.VmomiSupport import ManagedObject

from pyVmomi.vim import ExtendedElementDescription
from pyVmomi.vim import ManagedEntity
from pyVmomi.vim import Task

from pyVmomi.vmodl import DynamicData
from pyVmomi.vmodl import LocalizableMessage

class Profile(ManagedObject):
   class CreateSpec(DynamicData):
      name: Optional[str] = None
      annotation: Optional[str] = None
      enabled: Optional[bool] = None

   class SerializedCreateSpec(CreateSpec):
      profileConfigString: str

   class ConfigInfo(DynamicData):
      name: str
      annotation: Optional[str] = None
      enabled: bool

   class Description(DynamicData):
      class Section(DynamicData):
         description: ExtendedElementDescription
         message: list[LocalizableMessage] = []

      section: list[Section] = []

   @property
   def config(self) -> ConfigInfo: ...
   @property
   def description(self) -> Optional[Description]: ...
   @property
   def name(self) -> str: ...
   @property
   def createdTime(self) -> datetime: ...
   @property
   def modifiedTime(self) -> datetime: ...
   @property
   def entity(self) -> list[ManagedEntity]: ...
   @property
   def complianceStatus(self) -> str: ...

   def RetrieveDescription(self) -> Optional[Description]: ...
   def Destroy(self) -> NoReturn: ...
   def AssociateEntities(self, entity: list[ManagedEntity]) -> NoReturn: ...
   def DissociateEntities(self, entity: list[ManagedEntity]) -> NoReturn: ...
   def CheckCompliance(self, entity: list[ManagedEntity]) -> Task: ...
   def ExportProfile(self) -> str: ...



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import ClassVar
from typing import NoReturn
from typing import Optional

from pyVmomi.VmomiSupport import Enum
from pyVmomi.VmomiSupport import long

from pyVmomi.vim import Datastore
from pyVmomi.vim import EnvironmentBrowser
from pyVmomi.vim import Folder
from pyVmomi.vim import HostSystem
from pyVmomi.vim import HttpNfcLease
from pyVmomi.vim import ManagedEntity
from pyVmomi.vim import Network
from pyVmomi.vim import ResourceConfigSpec
from pyVmomi.vim import ResourcePool
from pyVmomi.vim import Task

from pyVmomi.vmodl import DynamicData
from pyVmomi.vmodl import MethodFault

from pyVmomi.vim.host import FeatureMask

from pyVmomi.vim.vm import Capability
from pyVmomi.vim.vm import CertThumbprint
from pyVmomi.vim.vm import CloneSpec
from pyVmomi.vim.vm import ConfigInfo
from pyVmomi.vim.vm import ConfigSpec
from pyVmomi.vim.vm import FaultToleranceConfigSpec
from pyVmomi.vim.vm import FileLayout
from pyVmomi.vim.vm import FileLayoutEx
from pyVmomi.vim.vm import GuestInfo
from pyVmomi.vim.vm import GuestQuiesceSpec
from pyVmomi.vim.vm import InstantCloneSpec
from pyVmomi.vim.vm import RelocateSpec
from pyVmomi.vim.vm import RuntimeInfo
from pyVmomi.vim.vm import Snapshot
from pyVmomi.vim.vm import SnapshotInfo
from pyVmomi.vim.vm import SnapshotSelectionSpec
from pyVmomi.vim.vm import StorageInfo
from pyVmomi.vim.vm import Summary
from pyVmomi.vim.vm import UsbScanCodeSpec

from pyVmomi.vim.vslm import ID

from pyVmomi.vim.vm.customization import Specification

from pyVmomi.vim.vm.device import VirtualDisk

class VirtualMachine(ManagedEntity):
   class StorageRequirement(DynamicData):
      datastore: Datastore
      freeSpaceRequiredInKb: long

   class PowerState(Enum):
      poweredOff: ClassVar['PowerState'] = 'poweredOff'
      poweredOn: ClassVar['PowerState'] = 'poweredOn'
      suspended: ClassVar['PowerState'] = 'suspended'

   class AppHeartbeatStatusType(Enum):
      appStatusGray: ClassVar['AppHeartbeatStatusType'] = 'appStatusGray'
      appStatusGreen: ClassVar['AppHeartbeatStatusType'] = 'appStatusGreen'
      appStatusRed: ClassVar['AppHeartbeatStatusType'] = 'appStatusRed'

   class ConnectionState(Enum):
      connected: ClassVar['ConnectionState'] = 'connected'
      disconnected: ClassVar['ConnectionState'] = 'disconnected'
      orphaned: ClassVar['ConnectionState'] = 'orphaned'
      inaccessible: ClassVar['ConnectionState'] = 'inaccessible'
      invalid: ClassVar['ConnectionState'] = 'invalid'

   class CryptoState(Enum):
      unlocked: ClassVar['CryptoState'] = 'unlocked'
      locked: ClassVar['CryptoState'] = 'locked'

   class MovePriority(Enum):
      lowPriority: ClassVar['MovePriority'] = 'lowPriority'
      highPriority: ClassVar['MovePriority'] = 'highPriority'
      defaultPriority: ClassVar['MovePriority'] = 'defaultPriority'

   class Ticket(DynamicData):
      ticket: str
      cfgFile: str
      host: Optional[str] = None
      port: Optional[int] = None
      sslThumbprint: Optional[str] = None
      certThumbprintList: list[CertThumbprint] = []
      sslCertificate: Optional[str] = None
      url: Optional[str] = None

   class MksTicket(DynamicData):
      ticket: str
      cfgFile: str
      host: Optional[str] = None
      port: Optional[int] = None
      sslThumbprint: Optional[str] = None

   class FaultToleranceState(Enum):
      notConfigured: ClassVar['FaultToleranceState'] = 'notConfigured'
      disabled: ClassVar['FaultToleranceState'] = 'disabled'
      enabled: ClassVar['FaultToleranceState'] = 'enabled'
      needSecondary: ClassVar['FaultToleranceState'] = 'needSecondary'
      starting: ClassVar['FaultToleranceState'] = 'starting'
      running: ClassVar['FaultToleranceState'] = 'running'

   class RecordReplayState(Enum):
      recording: ClassVar['RecordReplayState'] = 'recording'
      replaying: ClassVar['RecordReplayState'] = 'replaying'
      inactive: ClassVar['RecordReplayState'] = 'inactive'

   class NeedSecondaryReason(Enum):
      initializing: ClassVar['NeedSecondaryReason'] = 'initializing'
      divergence: ClassVar['NeedSecondaryReason'] = 'divergence'
      lostConnection: ClassVar['NeedSecondaryReason'] = 'lostConnection'
      partialHardwareFailure: ClassVar['NeedSecondaryReason'] = 'partialHardwareFailure'
      userAction: ClassVar['NeedSecondaryReason'] = 'userAction'
      checkpointError: ClassVar['NeedSecondaryReason'] = 'checkpointError'
      other: ClassVar['NeedSecondaryReason'] = 'other'

   class FaultToleranceType(Enum):
      unset: ClassVar['FaultToleranceType'] = 'unset'
      recordReplay: ClassVar['FaultToleranceType'] = 'recordReplay'
      checkpointing: ClassVar['FaultToleranceType'] = 'checkpointing'

   class Connection(DynamicData):
      label: str
      client: str
      userName: str

   class MksConnection(Connection):
      pass

   class TicketType(Enum):
      mks: ClassVar['TicketType'] = 'mks'
      device: ClassVar['TicketType'] = 'device'
      guestControl: ClassVar['TicketType'] = 'guestControl'
      dnd: ClassVar['TicketType'] = 'dnd'
      webmks: ClassVar['TicketType'] = 'webmks'
      guestIntegrity: ClassVar['TicketType'] = 'guestIntegrity'
      webRemoteDevice: ClassVar['TicketType'] = 'webRemoteDevice'

   class DisplayTopology(DynamicData):
      x: int
      y: int
      width: int
      height: int

   class DiskChangeInfo(DynamicData):
      class DiskChangeExtent(DynamicData):
         start: long
         length: long

      startOffset: long
      length: long
      changedArea: list[DiskChangeExtent] = []

   class WipeResult(DynamicData):
      diskId: int
      shrinkableDiskSpace: long

   @property
   def capability(self) -> Capability: ...
   @property
   def config(self) -> Optional[ConfigInfo]: ...
   @property
   def layout(self) -> Optional[FileLayout]: ...
   @property
   def layoutEx(self) -> Optional[FileLayoutEx]: ...
   @property
   def storage(self) -> Optional[StorageInfo]: ...
   @property
   def environmentBrowser(self) -> EnvironmentBrowser: ...
   @property
   def resourcePool(self) -> Optional[ResourcePool]: ...
   @property
   def parentVApp(self) -> Optional[ManagedEntity]: ...
   @property
   def resourceConfig(self) -> Optional[ResourceConfigSpec]: ...
   @property
   def runtime(self) -> RuntimeInfo: ...
   @property
   def guest(self) -> Optional[GuestInfo]: ...
   @property
   def summary(self) -> Summary: ...
   @property
   def datastore(self) -> list[Datastore]: ...
   @property
   def network(self) -> list[Network]: ...
   @property
   def snapshot(self) -> Optional[SnapshotInfo]: ...
   @property
   def rootSnapshot(self) -> list[Snapshot]: ...
   @property
   def guestHeartbeatStatus(self) -> ManagedEntity.Status: ...

   def RefreshStorageInfo(self) -> NoReturn: ...
   def CreateSnapshot(self, name: str, description: Optional[str], memory: bool, quiesce: bool) -> Task: ...
   def CreateSnapshotEx(self, name: str, description: Optional[str], memory: bool, quiesceSpec: Optional[GuestQuiesceSpec]) -> Task: ...
   def RevertToCurrentSnapshot(self, host: Optional[HostSystem], suppressPowerOn: Optional[bool]) -> Task: ...
   def RemoveAllSnapshots(self, consolidate: Optional[bool], spec: Optional[SnapshotSelectionSpec]) -> Task: ...
   def ConsolidateDisks(self) -> Task: ...
   def EstimateStorageRequirementForConsolidate(self) -> Task: ...
   def Reconfigure(self, spec: ConfigSpec) -> Task: ...
   def UpgradeVirtualHardware(self, version: Optional[str]) -> Task: ...
   def ExtractOvfEnvironment(self) -> str: ...
   def PowerOn(self, host: Optional[HostSystem]) -> Task: ...
   def PowerOff(self) -> Task: ...
   def Suspend(self) -> Task: ...
   def Reset(self) -> Task: ...
   def ShutdownGuest(self) -> NoReturn: ...
   def RebootGuest(self) -> NoReturn: ...
   def StandbyGuest(self) -> NoReturn: ...
   def Answer(self, questionId: str, answerChoice: str) -> NoReturn: ...
   def Customize(self, spec: Specification) -> Task: ...
   def CheckCustomizationSpec(self, spec: Specification) -> NoReturn: ...
   def Migrate(self, pool: Optional[ResourcePool], host: Optional[HostSystem], priority: MovePriority, state: Optional[PowerState]) -> Task: ...
   def Relocate(self, spec: RelocateSpec, priority: Optional[MovePriority]) -> Task: ...
   def Clone(self, folder: Folder, name: str, spec: CloneSpec) -> Task: ...
   def InstantClone(self, spec: InstantCloneSpec) -> Task: ...
   def ExportVm(self) -> HttpNfcLease: ...
   def MarkAsTemplate(self) -> NoReturn: ...
   def MarkAsVirtualMachine(self, pool: ResourcePool, host: Optional[HostSystem]) -> NoReturn: ...
   def Unregister(self) -> NoReturn: ...
   def ResetGuestInformation(self) -> NoReturn: ...
   def MountToolsInstaller(self) -> NoReturn: ...
   def UnmountToolsInstaller(self) -> NoReturn: ...
   def UpgradeTools(self, installerOptions: Optional[str]) -> Task: ...
   def AcquireMksTicket(self) -> MksTicket: ...
   def QueryConnections(self) -> list[Connection]: ...
   def DropConnections(self, listOfConnections: list[Connection]) -> bool: ...
   def AcquireTicket(self, ticketType: str) -> Ticket: ...
   def SetScreenResolution(self, width: int, height: int) -> NoReturn: ...
   def DefragmentAllDisks(self) -> NoReturn: ...
   def CreateSecondary(self, host: Optional[HostSystem]) -> Task: ...
   def CreateSecondaryEx(self, host: Optional[HostSystem], spec: Optional[FaultToleranceConfigSpec]) -> Task: ...
   def TurnOffFaultTolerance(self) -> Task: ...
   def MakePrimary(self, vm: VirtualMachine) -> Task: ...
   def TerminateFaultTolerantVM(self, vm: Optional[VirtualMachine]) -> Task: ...
   def DisableSecondary(self, vm: VirtualMachine) -> Task: ...
   def EnableSecondary(self, vm: VirtualMachine, host: Optional[HostSystem]) -> Task: ...
   def SetDisplayTopology(self, displays: list[DisplayTopology]) -> NoReturn: ...
   def StartRecording(self, name: str, description: Optional[str]) -> Task: ...
   def StopRecording(self) -> Task: ...
   def StartReplaying(self, replaySnapshot: Snapshot) -> Task: ...
   def StopReplaying(self) -> Task: ...
   def PromoteDisks(self, unlink: bool, disks: list[VirtualDisk]) -> Task: ...
   def CreateScreenshot(self) -> Task: ...
   def PutUsbScanCodes(self, spec: UsbScanCodeSpec) -> int: ...
   def QueryChangedDiskAreas(self, snapshot: Optional[Snapshot], deviceKey: int, startOffset: long, changeId: str) -> DiskChangeInfo: ...
   def QueryUnownedFiles(self) -> list[str]: ...
   def ReloadFromPath(self, configurationPath: str) -> Task: ...
   def QueryFaultToleranceCompatibility(self) -> list[MethodFault]: ...
   def QueryFaultToleranceCompatibilityEx(self, forLegacyFt: Optional[bool]) -> list[MethodFault]: ...
   def Terminate(self) -> NoReturn: ...
   def SendNMI(self) -> NoReturn: ...
   def AttachDisk(self, diskId: ID, datastore: Datastore, controllerKey: Optional[int], unitNumber: Optional[int]) -> Task: ...
   def DetachDisk(self, diskId: ID) -> Task: ...
   def ApplyEvcMode(self, mask: list[FeatureMask], completeMasks: Optional[bool]) -> Task: ...
   def CryptoUnlock(self) -> Task: ...



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import NoReturn
from typing import Optional

from pyVmomi.VmomiSupport import ManagedObject

from pyVmomi.vim.option import OptionDef
from pyVmomi.vim.option import OptionValue

class OptionManager(ManagedObject):
   @property
   def supportedOption(self) -> list[OptionDef]: ...
   @property
   def setting(self) -> list[OptionValue]: ...

   def QueryView(self, name: Optional[str]) -> list[OptionValue]: ...
   def UpdateValues(self, changedValue: list[OptionValue]) -> NoReturn: ...
